import logging

from django.db import connection
from django.db.models import BigIntegerField, BooleanField

from isc_common import setAttr
from isc_common.fields.name_field import NameField
from isc_common.fields.related import ForeignKeyProtect, ForeignKeySetNull
from isc_common.http.DSRequest import DSRequest
from isc_common.models.audit import AuditManager, AuditQuerySet, AuditModel
from isc_common.models.tree_audit import TreeAuditModelManager
from isc_common.number import DelProps
from isc_common.string import get_NoneStr
from kaf_pas.ckk.models.attr_type import Attr_type
from kaf_pas.kd.models.document_attributes import Document_attributes
from kaf_pas.kd.models.documents import Documents
from kaf_pas.kd.models.lotsman_documents_hierarcy import Lotsman_documents_hierarcyManager

logger = logging.getLogger(__name__)


class Lotsman_documents_hierarcy_viewQuerySet(AuditQuerySet):
    pass

    def get_range_rows5(self, start=None, end=None, function=None, json=None, distinct_field_names=None, *args, **kwargs):
        with connection.cursor() as cursor:
            cursor.execute(f'''select min(level)
            from kd_lotsman_documents_hierarcy_recurs_view
            where document_id = %s''', [kwargs.get('document_id')])
            level_min = cursor.fetchone()[0]

            if level_min == None:
                level_min = 1

            cursor.execute(f'''SELECT 'SELECT case when a.level = {level_min} then null
                                       else a.parent_id
                                       end as parent_id, ' || array_to_string(ARRAY(SELECT 'a' || '."' || c.column_name || '"'
                                                                                    FROM information_schema.columns As c
                                                                                    WHERE table_name = 'kd_lotsman_documents_hierarcy_recurs_view'
                                                                                      AND c.column_name NOT IN ('parent_id')
                                                                                         ), ',') || ' FROM kd_lotsman_documents_hierarcy_recurs_view as a where a.document_id=%s' as sqlstmt''', [kwargs.get('document_id')])

            sqlstmt = cursor.fetchone()[0]

            m_view_recurs_doc_name = f'''kd_lotsman_documents_hierarcy_recurs_{kwargs.get('document_id')}_view'''

            sqlstmt = f'''DROP VIEW IF EXISTS {m_view_recurs_doc_name};
                          CREATE VIEW {m_view_recurs_doc_name} AS {sqlstmt};'''
            cursor.execute(sqlstmt)

            parent_str = f'parent_id is null'
            if kwargs.get('parent_id') != None:
                parent_str = f'''parent_id={kwargs.get('parent_id')}'''

            sqlstmt = f'''select * from {m_view_recurs_doc_name} 
                           where {parent_str}   
                             limit %s offset %s'''

        queryResult = Lotsman_documents_hierarcy_view.objects.raw(raw_query=sqlstmt, params=[end - start, start])
        return [function(record) for record in queryResult]

    def get_range_rows4(self, request, function=None, distinct_field_names=None):
        request = DSRequest(request=request)
        criteria = request.get_criteria()
        criteria_dict = dict()

        for criterion in criteria:
            if isinstance(criterion, dict):
                criteria_dict.setdefault(criterion.get('fieldName'), criterion.get('value'))

        self.alive_only = request.alive_only
        self.enabledAll = request.enabledAll
        res = self.get_range_rows5(start=request.startRow, end=request.endRow, function=function, distinct_field_names=distinct_field_names, json=request.json, **criteria_dict)
        return res


class Lotsman_documents_hierarcy_viewManager(AuditManager):

    @staticmethod
    def getIcon(record):
        if record.attr_name != None:
            _section = record.attr_name.lower()
            if _section == 'документация':
                return 'documentation.png'
            elif _section == 'изделие':
                return 'product.png'
            elif _section == 'комплексы':
                return 'complexes.png'
            elif _section == 'сборочная единица':
                return 'folder_256.png'
            elif _section == 'деталь':
                return 'part.png'
            elif _section == 'стандартные изделия':
                return 'standard_prod.png'
            elif _section == 'прочие изделия':
                return 'other.png'
            elif _section == 'материалы':
                return 'materials.png'
            elif _section == 'комплекты':
                return 'kits.png'
            elif _section == 'папка':
                return 'folder_256.png'
            elif _section == 'материал':
                return 'material.png'
            elif _section == 'ведомость':
                return 'templates.png'
            else:
                return 'question.png'
        else:
            return 'question.png'

    @staticmethod
    def getRecord(record):
        def set_document_icon(record):
            from kaf_pas.kd.models.lotsman_documents_hierarcy_files import Lotsman_documents_hierarcy_files
            setAttr(record, 'isLoadedPict', Lotsman_documents_hierarcy_files.objects.filter(lotsman_document_id=record.get('id')).count() > 0)
            if record.get('isLoadedPict'):
                setAttr(record, 'icon', 'documentation.png')
            else:
                setAttr(record, 'icon', 'documentation_not_loaded.png')

        res = {
            'id': record.id,
            'parent_id': record.parent_id,
            'editing': record.editing,
            'deliting': record.deliting,
            'document_id': record.document.id,

            'STMP_120_id': record.STMP_120.id if record.STMP_120 else None,
            'STMP_120__value_str': record.STMP_120.value_str if record.STMP_120 else None,

            'SPC_CLM_FORMAT_id': record.SPC_CLM_FORMAT.id if record.SPC_CLM_FORMAT else None,
            'SPC_CLM_FORMAT__value_str': record.SPC_CLM_FORMAT.value_str if record.SPC_CLM_FORMAT else None,
            'SPC_CLM_ZONE_id': record.SPC_CLM_ZONE.id if record.SPC_CLM_ZONE else None,
            'SPC_CLM_ZONE__value_str': record.SPC_CLM_ZONE.value_str if record.SPC_CLM_ZONE else None,
            'SPC_CLM_POS_id': record.SPC_CLM_POS.id if record.SPC_CLM_POS else None,
            'SPC_CLM_POS__value_int': record.SPC_CLM_POS.value_int if record.SPC_CLM_POS else None,
            'SPC_CLM_MARK_id': record.SPC_CLM_MARK.id if record.SPC_CLM_MARK else None,
            'SPC_CLM_MARK__value_str': record.SPC_CLM_MARK.value_str if record.SPC_CLM_MARK else None,
            'SPC_CLM_NAME_id': record.SPC_CLM_NAME.id if record.SPC_CLM_NAME else None,
            'SPC_CLM_NAME__value_str': record.SPC_CLM_NAME.value_str if record.SPC_CLM_NAME else None,
            'SPC_CLM_COUNT_id': record.SPC_CLM_COUNT.id if record.SPC_CLM_COUNT else None,
            'SPC_CLM_COUNT__value_str': record.SPC_CLM_COUNT.value_str if record.SPC_CLM_COUNT else None,
            'SPC_CLM_NOTE_id': record.SPC_CLM_NOTE.id if record.SPC_CLM_NOTE else None,
            'SPC_CLM_NOTE__value_str': record.SPC_CLM_NOTE.value_str if record.SPC_CLM_NOTE else None,
            'SPC_CLM_MASSA_id': record.SPC_CLM_MASSA.id if record.SPC_CLM_MASSA else None,
            'SPC_CLM_MASSA__value_str': record.SPC_CLM_MASSA.value_str if record.SPC_CLM_MASSA else None,
            'SPC_CLM_MATERIAL_id': record.SPC_CLM_MATERIAL.id if record.SPC_CLM_MATERIAL else None,
            'SPC_CLM_MATERIAL__value_str': record.SPC_CLM_MATERIAL.value_str if record.SPC_CLM_MATERIAL else None,
            'SPC_CLM_USER_id': record.SPC_CLM_USER.id if record.SPC_CLM_USER else None,
            'SPC_CLM_USER__value_str': record.SPC_CLM_USER.value_str if record.SPC_CLM_USER else None,
            'SPC_CLM_KOD_id': record.SPC_CLM_KOD.id if record.SPC_CLM_KOD else None,
            'SPC_CLM_KOD__value_str': record.SPC_CLM_KOD.value_str if record.SPC_CLM_KOD else None,
            'SPC_CLM_FACTORY_id': record.SPC_CLM_FACTORY.id if record.SPC_CLM_FACTORY else None,
            'SPC_CLM_FACTORY__value_str': record.SPC_CLM_FACTORY.value_str if record.SPC_CLM_FACTORY else None,
            'Наименование_материала_id': record.Наименование_материала.id if record.Наименование_материала else None,
            'Наименование_материала__value_str': record.Наименование_материала.value_str if record.Наименование_материала else None,
            'Документ_на_материал_id': record.Документ_на_материал.value_str if record.Документ_на_материал else None,
            'Наименование_сортамента__value_str': record.Наименование_сортамента.value_str if record.Наименование_сортамента else None,
            'Документ_на_сортамент_id': record.Документ_на_сортамент.value_str if record.Документ_на_сортамент else None,
            'Документ_на_сортамент__value_str': record.Документ_на_сортамент.value_str if record.Документ_на_сортамент else None,
            '_Version_id': record._Version.value_int if record._Version else None,
            '_Version__value_int': record._Version.value_int if record._Version else None,
            # '_Type_id': record.__Type.value_int if record._Type else None,
            # '_Type__value_int': record._Type.value_int if record._Type else None,

            'section': record.section,
            'subsection': record.subsection,
            'attr_code': record.attr_code,
            'attr_name': record.attr_name,
            'isFolder': record.isFolder,
            'isLoadedPict': False,
            'icon': Lotsman_documents_hierarcy_viewManager.getIcon(record)
        }
        res = DelProps(res)

        _section = record.attr_name.lower()
        if _section == 'спецификация':
            set_document_icon(res)

        elif _section == 'чертеж':
            set_document_icon(res)

        # if res.get('isLoadedPict') == True:
        #     print(res, end='\n\n')
        return res

    def get_queryset(self):
        return Lotsman_documents_hierarcy_viewQuerySet(self.model, using=self._db)


class Lotsman_documents_hierarcy_view(AuditModel):
    parent_id = BigIntegerField(null=True, blank=True)
    attr_type = ForeignKeyProtect(Attr_type, verbose_name='Тип документа')
    document = ForeignKeySetNull(Documents, null=True, blank=True)

    STMP_120 = ForeignKeyProtect(Document_attributes, related_name='STMP_120_Lotsman_documents_hierarcy_view', null=True, blank=True)

    SPC_CLM_KOD = ForeignKeyProtect(Document_attributes, related_name='SPC_CLM_KOD_Lotsman_documents_hierarcy_view', null=True, blank=True)
    SPC_CLM_FORMAT = ForeignKeyProtect(Document_attributes, related_name='SPC_CLM_FORMAT_Lotsman_documents_hierarcy_view', null=True, blank=True)
    SPC_CLM_COUNT = ForeignKeyProtect(Document_attributes, related_name='SPC_CLM_COUNT_Lotsman_documents_hierarcy_view', null=True, blank=True)
    SPC_CLM_ZONE = ForeignKeyProtect(Document_attributes, related_name='SPC_CLM_ZONE_Lotsman_documents_hierarcy_view', null=True, blank=True)
    SPC_CLM_MASSA = ForeignKeyProtect(Document_attributes, related_name='SPC_CLM_MASSA_Lotsman_documents_hierarcy_view', null=True, blank=True)
    SPC_CLM_POS = ForeignKeyProtect(Document_attributes, related_name='SPC_CLM_POS_Lotsman_documents_hierarcy_view', null=True, blank=True)
    SPC_CLM_NAME = ForeignKeyProtect(Document_attributes, related_name='SPC_CLM_NAME_Lotsman_documents_hierarcy_view', null=True, blank=True)
    SPC_CLM_MARK = ForeignKeyProtect(Document_attributes, related_name='SPC_CLM_MARK_Lotsman_documents_hierarcy_view', null=True, blank=True)
    SPC_CLM_NOTE = ForeignKeyProtect(Document_attributes, related_name='SPC_CLM_NOTE_Lotsman_documents_hierarcy_view', null=True, blank=True)
    SPC_CLM_USER = ForeignKeyProtect(Document_attributes, related_name='SPC_CLM_USER_Lotsman_documents_hierarcy_view', null=True, blank=True)
    SPC_CLM_FACTORY = ForeignKeyProtect(Document_attributes, related_name='SPC_CLM_FACTORY_Lotsman_documents_hierarcy_view', null=True, blank=True)
    SPC_CLM_MATERIAL = ForeignKeyProtect(Document_attributes, related_name='SPC_CLM_MATERIAL_Lotsman_documents_hierarcy_view', null=True, blank=True)

    Документ_на_материал = ForeignKeyProtect(Document_attributes, related_name='Документ_на_материал_Lotsman_documents_hierarcy_view', null=True, blank=True)
    Наименование_материала = ForeignKeyProtect(Document_attributes, related_name='Наименование_материала_Lotsman_documents_hierarcy_view', null=True, blank=True)
    Документ_на_сортамент = ForeignKeyProtect(Document_attributes, related_name='Документ_на_сортамент_Lotsman_documents_hierarcy_view', null=True, blank=True)
    Наименование_сортамента = ForeignKeyProtect(Document_attributes, related_name='Наименование_сортамента_Lotsman_documents_hierarcy_view', null=True, blank=True)
    _Version = ForeignKeyProtect(Document_attributes, related_name='_Version_Lotsman_documents_hierarcy_view', null=True, blank=True)
    _Type = ForeignKeyProtect(Document_attributes, related_name='_Type_id_Lotsman_documents_hierarcy_view', null=True, blank=True)

    section = NameField(null=True, blank=True)
    subsection = NameField(null=True, blank=True)
    attr_code = NameField(null=True, blank=True)
    attr_name = NameField(null=True, blank=True)
    isFolder = BooleanField(null=True, blank=True)
    props = Lotsman_documents_hierarcyManager.get_props()

    objects = Lotsman_documents_hierarcy_viewManager()
    objects_tree = TreeAuditModelManager()

    def __str__(self):
        return f'ID:{self.id}, \n' \
               f'attr_type: [{self.attr_type}], \n' \
               f'document: [{self.document}], \n' \
               f'STMP_120: [{self.STMP_120}], \n' \
               f'SPC_CLM_KOD: [{self.SPC_CLM_KOD}], \n' \
               f'SPC_CLM_FORMAT: [{self.SPC_CLM_FORMAT}], \n' \
               f'SPC_CLM_COUNT: [{self.SPC_CLM_COUNT}], \n' \
               f'SPC_CLM_ZONE: [{self.SPC_CLM_ZONE}], \n' \
               f'SPC_CLM_MASSA: [{self.SPC_CLM_MASSA}], \n' \
               f'SPC_CLM_POS: [{self.SPC_CLM_POS}], \n' \
               f'SPC_CLM_NAME: [{self.SPC_CLM_NAME}], \n' \
               f'SPC_CLM_MARK: [{self.SPC_CLM_MARK}], \n' \
               f'SPC_CLM_NOTE: [{self.SPC_CLM_NOTE}], \n' \
               f'SPC_CLM_USER: [{self.SPC_CLM_USER}], \n' \
               f'SPC_CLM_FACTORY: [{self.SPC_CLM_FACTORY}], \n' \
               f'SPC_CLM_MATERIAL: [{self.SPC_CLM_MATERIAL}], \n' \
               f'Документ_на_материал: [{self.Документ_на_материал}], \n' \
               f'Наименование_материала: [{self.Наименование_материала}], \n' \
               f'Документ_на_сортамент: [{self.Документ_на_сортамент}], \n' \
               f'Наименование_сортамента: [{self.Наименование_сортамента}], \n' \
               f'_Version: [{self._Version}], \n' \
               f'_Type: [{self._Type}], \n' \
               f'section: {self.section}, \n' \
               f'subsection: {self.subsection}, \n' \
               f'attr_code: {self.attr_code}, \n' \
               f'attr_name: {self.attr_name}, \n' \
               f'isFolder: {self.isFolder}, \n' \
               f'props: {self.props} \n'

    def __repr__(self):
        return self.__str__()

    def get_mat(selfname, name, doc):
        if name != None and doc != None:
            return f'{name}/{doc}'
        elif name != None and doc == None:
            return name
        elif name == None and doc != None:
            return doc
        else:
            return None

    @property
    def item_name(self):
        if self.SPC_CLM_NAME != None and self.SPC_CLM_MARK != None:
            return f'{get_NoneStr(self.SPC_CLM_NAME.value_str)}: {get_NoneStr(self.SPC_CLM_MARK.value_str)}'
        elif self.SPC_CLM_NAME == None and self.SPC_CLM_MARK != None:
            return get_NoneStr(self.SPC_CLM_MARK.value_str)
        elif self.SPC_CLM_NAME != None and self.SPC_CLM_MARK == None:
            return get_NoneStr(self.SPC_CLM_NAME.value_str)
        else:
            return 'Неизвестен'

    @property
    def SPC_CLM_MATERIAL_NAME_DOC(self):
        name = f'{self.Наименование_материала.value_str if self.Наименование_материала.value_str else None}'
        doc = f'{self.Документ_на_материал.value_str if self.Документ_на_материал.value_str else None}'

        return self.get_mat(name=name, doc=doc)

    @property
    def SPC_CLM_MATERIAL_SORT_DOC(self):
        name = f'{self.Наименование_сортамента.value_str if self.Наименование_сортамента.value_str else None}'
        doc = f'{self.Документ_на_сортамент.value_str if self.Документ_на_сортамент.value_str else None}'

        return self.get_mat(name=name, doc=doc)

    class Meta:
        verbose_name = 'Иерархия документа из Лоцмана'
        managed = False
        db_table = 'kd_lotsman_documents_hierarcy_mview'



from django.urls import path

from kaf_pas.kd.views import document_attributes_SPC_CLM_FACTORY

urlpatterns = [

    path('Document_attributes_SPC_CLM_FACTORY/Fetch/', document_attributes_SPC_CLM_FACTORY.Document_attributes_SPC_CLM_FACTORY_Fetch),
    path('Document_attributes_SPC_CLM_FACTORY/Add', document_attributes_SPC_CLM_FACTORY.Document_attributes_SPC_CLM_FACTORY_Add),
    path('Document_attributes_SPC_CLM_FACTORY/Update', document_attributes_SPC_CLM_FACTORY.Document_attributes_SPC_CLM_FACTORY_Update),
    path('Document_attributes_SPC_CLM_FACTORY/Remove', document_attributes_SPC_CLM_FACTORY.Document_attributes_SPC_CLM_FACTORY_Remove),
    path('Document_attributes_SPC_CLM_FACTORY/Lookup/', document_attributes_SPC_CLM_FACTORY.Document_attributes_SPC_CLM_FACTORY_Lookup),
    path('Document_attributes_SPC_CLM_FACTORY/Info/', document_attributes_SPC_CLM_FACTORY.Document_attributes_SPC_CLM_FACTORY_Info),
    path('Document_attributes_SPC_CLM_FACTORY/Copy', document_attributes_SPC_CLM_FACTORY.Document_attributes_SPC_CLM_FACTORY_Copy),

]



import logging

from django.core.management import BaseCommand

from kaf_pas.kd.models.lotsman_documents_hierarcy_ext import Lotsman_documents_hierarcyManagerExt

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = "Тестирование get_m_view"

    def handle(self, *args, **options):
        logger.info(self.help)

        Lotsman_documents_hierarcyManagerExt(user=2).make_mview()



from django.urls import path

from kaf_pas.production.views import launch_operations_item, operations_item

urlpatterns = [

    path('Launch_operations_item/Fetch/', launch_operations_item.Launch_operations_item_Fetch),
    path('Launch_operations_item/Add', launch_operations_item.Launch_operations_item_Add),
    path('Launch_operations_item/Update', launch_operations_item.Launch_operations_item_Update),
    path('Launch_operations_item/Remove', launch_operations_item.Launch_operations_item_Remove),
    path('Launch_operations_item/Lookup/', launch_operations_item.Launch_operations_item_Lookup),
    path('Launch_operations_item/Info/', launch_operations_item.Launch_operations_item_Info),
    path('Launch_operations_item/Copy', launch_operations_item.Launch_operations_item_Copy),
    path('Launch_operations_item/CopyOpers', launch_operations_item.Launch_operations_item_CopyOpers),

]



# Generated by Django 2.2.1 on 2019-05-29 13:41

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0069_materials_material_1c'),
        ('production', '0038_resource_categories'),
    ]

    operations = [
        migrations.AddField(
            model_name='resource',
            name='category',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='production.Resource_categories'),
        ),
        migrations.AddField(
            model_name='resource',
            name='location',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='ckk.Locations'),
        ),
    ]



import logging

from bitfield import BitField
from django.conf import settings
from django.db import transaction
from django.db.models import BigIntegerField, deletion

from isc_common.fields.related import ForeignKeyProtect
from isc_common.http.DSRequest import DSRequest
from isc_common.logger.Logger import Logger
from isc_common.models.audit import AuditManager, AuditQuerySet, AuditModel
from kaf_pas.ckk.models.attr_type import Attr_type
from kaf_pas.kd.models.documents import Documents

logger = logging.getLogger(__name__)
logger.__class__ = Logger


class Lotsman_documents_hierarcyQuerySet(AuditQuerySet):
    pass


class Lotsman_documents_hierarcyManager(AuditManager):

    @staticmethod
    def delete(id, user):
        from isc_common.auth.models.user import User
        from kaf_pas.ckk.models.item import Item
        from kaf_pas.ckk.models.item import ItemManager
        from kaf_pas.kd.models.documents_thumb import Documents_thumb
        from kaf_pas.kd.models.documents_thumb10 import Documents_thumb10
        from kaf_pas.kd.models.lotsman_document_attr_cross import Lotsman_document_attr_cross
        from kaf_pas.kd.models.lotsman_documents_hierarcy_files import Lotsman_documents_hierarcy_files

        if not isinstance(user, User):
            raise Exception(f'user  must be a User instance.')

        res = 0

        settings.LOCKS.acquire('Lotsman_documents_hierarcyManager.delete')
        Documents_thumb.objects.filter(lotsman_document_id=id).delete()
        Documents_thumb10.objects.filter(lotsman_document_id=id).delete()
        Lotsman_document_attr_cross.objects.filter(document_id=id).delete()

        for item in Item.objects.filter(lotsman_document_id=id):
            ItemManager.delete_recursive(item_id=item.id, user=user)

        Lotsman_documents_hierarcy_files.objects.filter(lotsman_document_id=id).delete()
        try:
            res += Lotsman_documents_hierarcy.objects.filter(id=id).delete()[0]
        except deletion.ProtectedError:
            pass
        settings.LOCKS.release('Lotsman_documents_hierarcyManager.delete')
        return res

    @staticmethod
    def delete_file(id, user):
        from isc_common.auth.models.user import User
        from kaf_pas.kd.models.documents_thumb import Documents_thumb
        from kaf_pas.kd.models.documents_thumb10 import Documents_thumb10
        from kaf_pas.kd.models.lotsman_documents_hierarcy_files import Lotsman_documents_hierarcy_files
        from django.conf import settings

        settings.LOCKS.acquire('Lotsman_documents_hierarcyManager.delete_file')

        if not isinstance(user, User):
            raise Exception(f'user  must be a User instance.')

        res = 0

        res += Documents_thumb.objects.filter(lotsman_document_id=id).delete()[0]
        res += Documents_thumb10.objects.filter(lotsman_document_id=id).delete()[0]

        res += Lotsman_documents_hierarcy_files.objects.filter(lotsman_document_id=id).delete()[0]
        settings.LOCKS.release('Lotsman_documents_hierarcyManager.delete_file')
        return res

    @staticmethod
    def get_props():
        return BitField(flags=(
            ('relevant', 'Актуальность'),
            ('beenItemed', 'Был внесен в состав изделий'),
        ), default=1, db_index=True)

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def makeItemFromRequest(self, request):
        from kaf_pas.kd.models.lotsman_documents_hierarcy_view import Lotsman_documents_hierarcy_view
        from kaf_pas.kd.models.lotsman_documents_hierarcy_ext import Lotsman_documents_hierarcyManagerExt
        from kaf_pas.kd.models.documents_ext import DocumentManagerExt

        request = DSRequest(request=request)
        data = request.get_data()

        Lotsman_documents_hierarcyManagerExt(
            user=request.user,
            documentManagerExt=DocumentManagerExt(
                logger=logger
            )
        ).make_items(
            lotsman_documents_hierarcy_view_record=Lotsman_documents_hierarcy_view.objects.get(id=data.get('id'))
        )

        return data

    def get_queryset(self):
        return Lotsman_documents_hierarcyQuerySet(self.model, using=self._db)

    def deleteFromRequest(self, request, removed=None, ):
        request = DSRequest(request=request)
        res = 0
        tuple_ids = request.get_tuple_ids()
        with transaction.atomic():
            for id, mode in tuple_ids:
                if mode == 'hide':
                    super().filter(id=id).soft_delete()
                else:
                    qty, _ = super().filter(id=id).delete()
                res += qty
        return res


class Lotsman_documents_hierarcy(AuditModel):
    id = BigIntegerField(primary_key=True, verbose_name="Идентификатор")
    attr_type = ForeignKeyProtect(Attr_type, verbose_name='Тип документа')
    document = ForeignKeyProtect(Documents)

    props = Lotsman_documents_hierarcyManager.get_props()

    objects = Lotsman_documents_hierarcyManager()

    def __str__(self):
        return f'ID:{self.id}, attr_type: [{self.attr_type}], document: [{self.document}], props: {self.props}'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Иерархия документа из Лоцмана'



from django.urls import path

from kaf_pas.ckk.views import material_type
from kaf_pas.production.views import operations_item

urlpatterns = [

    path('Operations_item/Fetch/', operations_item.Operations_item_Fetch),
    path('Operations_item/Add', operations_item.Operations_item_Add),
    path('Operations_item/Update', operations_item.Operations_item_Update),
    path('Operations_item/Remove', operations_item.Operations_item_Remove),
    path('Operations_item/Lookup/', operations_item.Operations_item_Lookup),
    path('Operations_item/Info/', operations_item.Operations_item_Info),
    path('Operations_item/Copy', operations_item.Operations_item_Copy),
    path('Operations_item/CopyOpers', operations_item.Operations_item_CopyOpers),

]



# Generated by Django 2.2.1 on 2019-05-23 17:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0026_precent_items_qty'),
    ]

    operations = [
        migrations.AlterField(
            model_name='precent_items',
            name='qty',
            field=models.PositiveIntegerField(),
        ),
    ]



import logging

from django.core.management import BaseCommand

from kaf_pas.production.models.launch_operation_resources import Launch_operation_resourcesManager
from kaf_pas.production.models.operation_resources import Operation_resourcesManager

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = "Тестирование"

    def handle(self, *args, **options):
        Launch_operation_resourcesManager.refresh_resource()
        Operation_resourcesManager.refresh_resource()



import logging

from django.db.models import PositiveIntegerField
from isc_common.fields.name_field import NameField
from isc_common.fields.related import ForeignKeyProtect, ForeignKeyCascade
from isc_common.models.audit import AuditModel, AuditManager

from kaf_pas.ckk.models.item import Item_add, Item
from kaf_pas.kd.models.document_attributes import Document_attributes

logger = logging.getLogger(__name__)


class Item_line_viewManager(AuditManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'child_id': record.child.id if record.child else None,
            'parent_id': record.parent.id,
            'SPC_CLM_FORMAT_id': record.SPC_CLM_FORMAT.id if record.SPC_CLM_FORMAT else None,
            'SPC_CLM_FORMAT__value_str': record.SPC_CLM_FORMAT.value_str if record.SPC_CLM_FORMAT else None,
            'SPC_CLM_ZONE_id': record.SPC_CLM_ZONE.id if record.SPC_CLM_ZONE else None,
            'SPC_CLM_ZONE__value_str': record.SPC_CLM_ZONE.value_str if record.SPC_CLM_ZONE else None,
            'SPC_CLM_POS_id': record.SPC_CLM_POS.id if record.SPC_CLM_POS else None,
            'SPC_CLM_POS__value_str': record.SPC_CLM_POS.value_str if record.SPC_CLM_POS else None,
            'SPC_CLM_POS__value_int': record.SPC_CLM_POS.value_int if record.SPC_CLM_POS else None,
            'SPC_CLM_MARK_id': record.SPC_CLM_MARK.id if record.SPC_CLM_MARK else None,
            'SPC_CLM_MARK__value_str': record.SPC_CLM_MARK.value_str if record.SPC_CLM_MARK else None,
            'SPC_CLM_NAME_id': record.SPC_CLM_NAME.id if record.SPC_CLM_NAME else None,
            'SPC_CLM_NAME__value_str': record.SPC_CLM_NAME.value_str if record.SPC_CLM_NAME else None,
            'SPC_CLM_COUNT_id': record.SPC_CLM_COUNT.id if record.SPC_CLM_COUNT else None,
            'SPC_CLM_COUNT__value_str': record.SPC_CLM_COUNT.value_str if record.SPC_CLM_COUNT else None,
            'SPC_CLM_NOTE_id': record.SPC_CLM_NOTE.id if record.SPC_CLM_NOTE else None,
            'SPC_CLM_NOTE__value_str': record.SPC_CLM_NOTE.value_str if record.SPC_CLM_NOTE else None,
            'SPC_CLM_MASSA_id': record.SPC_CLM_MASSA.id if record.SPC_CLM_MASSA else None,
            'SPC_CLM_MASSA__value_str': record.SPC_CLM_MASSA.value_str if record.SPC_CLM_MASSA else None,
            'SPC_CLM_MATERIAL_id': record.SPC_CLM_MATERIAL.id if record.SPC_CLM_MATERIAL else None,
            'SPC_CLM_MATERIAL__value_str': record.SPC_CLM_MATERIAL.value_str if record.SPC_CLM_MATERIAL else None,
            'SPC_CLM_USER_id': record.SPC_CLM_USER.id if record.SPC_CLM_USER else None,
            'SPC_CLM_USER__value_str': record.SPC_CLM_USER.value_str if record.SPC_CLM_USER else None,
            'SPC_CLM_KOD_id': record.SPC_CLM_KOD.id if record.SPC_CLM_KOD else None,
            'SPC_CLM_KOD__value_str': record.SPC_CLM_KOD.value_str if record.SPC_CLM_KOD else None,
            'SPC_CLM_FACTORY_id': record.SPC_CLM_FACTORY.id if record.SPC_CLM_FACTORY else None,
            'SPC_CLM_FACTORY__value_str': record.SPC_CLM_FACTORY.value_str if record.SPC_CLM_FACTORY else None,
            'section': record.section,
            'section_num': record.section_num,
            'subsection': record.subsection,
            'where_from': record.where_from,
            'lastmodified': record.lastmodified,
            'editing': record.editing,
            'deliting': record.deliting,
            'item_props': record.item_props._value,
            'relevant': record.relevant,
            'confirmed': record.confirmed,
        }
        return res


class Item_line_view(AuditModel):
    parent = ForeignKeyProtect(Item, verbose_name='Товарная позиция', related_name='item_parent_v')
    child = ForeignKeyCascade(Item, verbose_name='Товарная позиция', related_name='item_child_v')

    SPC_CLM_FORMAT = ForeignKeyProtect(Document_attributes, verbose_name='Формат', related_name='SPC_CLM_FORMAT_v', null=True, blank=True)
    SPC_CLM_ZONE = ForeignKeyProtect(Document_attributes, verbose_name='Зона', related_name='SPC_CLM_ZONE_v', null=True, blank=True)
    SPC_CLM_POS = ForeignKeyProtect(Document_attributes, verbose_name='Позиция', related_name='SPC_CLM_POS_v', null=True, blank=True)
    SPC_CLM_MARK = ForeignKeyProtect(Document_attributes, verbose_name='Обозначение', related_name='SPC_CLM_MARK_v', null=True, blank=True)
    SPC_CLM_NAME = ForeignKeyProtect(Document_attributes, verbose_name='Наименование', related_name='SPC_CLM_NAME_v', null=True, blank=True)
    SPC_CLM_COUNT = ForeignKeyProtect(Document_attributes, verbose_name='Количество', related_name='SPC_CLM_COUNT_v', null=True, blank=True)
    SPC_CLM_NOTE = ForeignKeyProtect(Document_attributes, verbose_name='Примечание', related_name='SPC_CLM_NOTE_v', null=True, blank=True)
    SPC_CLM_MASSA = ForeignKeyProtect(Document_attributes, verbose_name='Масса', related_name='SPC_CLM_MASSA_v', null=True, blank=True)
    SPC_CLM_MATERIAL = ForeignKeyProtect(Document_attributes, verbose_name='Материал', related_name='SPC_CLM_MATERIAL_v', null=True, blank=True)
    SPC_CLM_USER = ForeignKeyProtect(Document_attributes, verbose_name='Пользовательская', related_name='SPC_CLM_USER_v', null=True, blank=True)
    SPC_CLM_KOD = ForeignKeyProtect(Document_attributes, verbose_name='Код', related_name='SPC_CLM_KOD_v', null=True, blank=True)
    SPC_CLM_FACTORY = ForeignKeyProtect(Document_attributes, verbose_name='Предприятие - изготовитель', related_name='SPC_CLM_FACTORY_v', null=True, blank=True)
    section = NameField()
    section_num = PositiveIntegerField(default=0)
    subsection = NameField()

    where_from = NameField()
    item_props = Item_add.get_prop_field()
    relevant = NameField()
    confirmed = NameField()

    objects = Item_line_viewManager()

    class Meta:
        verbose_name = 'Строка состава изделия'
        db_table = 'ckk_item_line_view'
        managed = False



# Generated by Django 2.2.6 on 2019-10-23 21:43

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0088_auto_20191023_1119'),
    ]

    operations = [
        migrations.RenameField(
            model_name='precent_dogovors',
            old_name='document',
            new_name='dogovor',
        ),
    ]



# Generated by Django 3.0.2 on 2020-01-31 11:05

import django.db.models.expressions
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('ckk', '0189_auto_20200131_1059'),
    ]

    operations = [
        migrations.RunSQL('''delete from ckk_item_line where parent_id = child_id '''),
        migrations.AddConstraint(
            model_name='item_line',
            constraint=models.CheckConstraint(check=models.Q(_negated=True, child=django.db.models.expressions.F('parent')), name='Item_line_not_circulate_refs'),
        ),
    ]



# Generated by Django 2.2.6 on 2019-10-22 22:24

import django.db.models.deletion
from django.db import migrations

import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0085_demand_precent'),
    ]

    operations = [
        migrations.AlterField(
            model_name='demand',
            name='precent',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='sales.Precent'),
        ),
    ]



import logging
import time

from django.conf import settings
from django.db import transaction
from django.db.models import DateTimeField, PositiveIntegerField

from isc_common import setAttr
from isc_common.bit import TurnBitOn
from isc_common.common import blinkString
from isc_common.datetime import DateToStr, StrToDate
from isc_common.fields.related import ForeignKeyProtect
from isc_common.http.DSRequest import DSRequest
from isc_common.models.base_ref import BaseRefHierarcy, BaseRefManager, BaseRefQuerySet
from isc_common.number import StrToNumber, DelProps
from isc_common.progress import managed_progress, ProgressDroped
from kaf_pas.production.models import progress_deleted
from kaf_pas.production.models.status_launch import Status_launch
from kaf_pas.sales.models.demand import Demand
from kaf_pas.sales.models.demand_view import Demand_view

logger = logging.getLogger(__name__)
logger_timing = logging.getLogger(f'{__name__}_timing')


class LaunchesQuerySet(BaseRefQuerySet):
    pass


class LaunchesManager(BaseRefManager):

    @staticmethod
    def get_count(item_line, qty, level, get_full_path):
        from kaf_pas.ckk.models.attr_type import Attr_type
        settings.LOCKS.acquire('LaunchesManager.get_count')
        SPC_CLM_COUNT_ATTR, _ = Attr_type.objects.get_or_create(code='SPC_CLM_COUNT')
        from kaf_pas.kd.models.document_attributes import Document_attributes

        res = None
        if level == 1:
            res, _ = Document_attributes.objects.get_or_create(attr_type=SPC_CLM_COUNT_ATTR, value_str=str(qty), value_int=qty)
            return res

        _str = item_line.SPC_CLM_COUNT.value_str if item_line.SPC_CLM_COUNT else None
        if _str != None:
            if _str.find(',') != -1:
                try:
                    str1 = _str.replace(',', '.')
                    count = StrToNumber(str1)

                    SPC_CLM_COUNT, created = Document_attributes.objects.get_or_create(attr_type=SPC_CLM_COUNT_ATTR, value_str=str1)
                    if created == False:
                        SPC_CLM_COUNT.value_int = count
                        SPC_CLM_COUNT.value_str = str1
                        SPC_CLM_COUNT.save()

                    if qty > 1:
                        count *= qty
                        SPC_CLM_COUNT, created = Document_attributes.objects.get_or_create(attr_type=SPC_CLM_COUNT_ATTR, value_str=str(count))
                        if created != True:
                            SPC_CLM_COUNT.value_int = count
                            SPC_CLM_COUNT.save()

                    res = SPC_CLM_COUNT
                except ValueError:
                    SPC_CLM_COUNT, created = Document_attributes.objects.get_or_create(attr_type=SPC_CLM_COUNT_ATTR, value_str='1')
                    if created == True:
                        SPC_CLM_COUNT.value_int = 1
                        SPC_CLM_COUNT.save()

            else:
                try:
                    count = StrToNumber(_str)
                    if qty > 1:
                        count *= qty
                        SPC_CLM_COUNT, created = Document_attributes.objects.get_or_create(attr_type=SPC_CLM_COUNT_ATTR, value_str=str(count))
                        if created == True:
                            SPC_CLM_COUNT.value_int = count
                            SPC_CLM_COUNT.save()
                        res = SPC_CLM_COUNT
                    else:
                        res = item_line.SPC_CLM_COUNT
                except ValueError:
                    SPC_CLM_COUNT, created = Document_attributes.objects.get_or_create(attr_type=SPC_CLM_COUNT_ATTR, value_str='1')
                    if created == True:
                        SPC_CLM_COUNT.value_int = 1
                        SPC_CLM_COUNT.save()

        else:
            res = item_line.SPC_CLM_COUNT
        settings.LOCKS.release('LaunchesManager.get_count')

        if res == None and item_line.section != 'Документация':
            raise Exception(blinkString(f'Для: {get_full_path} count == 0', color='red', bold=True, blink=False))
        return res

    @staticmethod
    def make_launch(data):
        # logger.debug(f'data: {data}')

        from isc_common.auth.models.user import User
        from isc_common.models.deleted_progresses import Deleted_progresses
        from kaf_pas.ckk.models.item_line import Item_line
        from kaf_pas.ckk.models.item_refs import Item_refs
        from kaf_pas.production.models import progress_deleted, p_id
        from kaf_pas.production.models.launch_item_line import Launch_item_line
        from kaf_pas.production.models.launch_item_refs import Launch_item_refs
        from kaf_pas.production.models.launch_operation_material import Launch_operations_material
        from kaf_pas.production.models.launch_operation_resources import Launch_operation_resources
        from kaf_pas.production.models.launch_operations_item import Launch_operations_item
        from kaf_pas.production.models.operation_material import Operation_material
        from kaf_pas.production.models.operation_resources import Operation_resources
        from kaf_pas.production.models.operations_item import Operations_item
        from kaf_pas.production.models.ready_2_launch import Item_refs_Stack

        demand_id = data.get('demand_id')
        qty = data.get('qty')

        description = data.get('description')
        date = data.get('date')
        date = StrToDate(date, '%Y-%m-%d')
        user = User.objects.get(id=data.get('user_id'))
        status = settings.PROD_OPERS_STACK.FORMIROVANIE
        step_res = 0

        try:
            settings.LOCKS.acquire('LaunchesManager.make_launch')
            with transaction.atomic():
                for demand in Demand.objects.select_for_update().filter(id=demand_id):
                    cnt = Launches.objects.filter(demand=demand).count() + 1
                    code = f'{demand.code}/{cnt}'
                    tail_qty = Demand_view.objects.get(id=demand_id).tail_qty
                    if tail_qty < qty:
                        raise Exception(f'Затребованное количестово для запуска ({qty}), превышает возможное к запуску ({tail_qty})')

                    launch = Launches.objects.create(
                        code=code,
                        date=date,
                        demand_id=demand_id,
                        description=description,
                        qty=qty,
                        status=status
                    )

                    where_clause = f'where is_bit_on(props::integer, 0) = true and is_bit_on(props::integer, 1) = true and parent_id != {p_id}'
                    cntAll = Item_refs.objects.get_descendants_count(
                        id=demand.precent_item.item.id,
                        where_clause=where_clause)

                    first_step = True

                    start_time = time.clock()
                    logger_timing.debug(f'Start Time: {start_time}')
                    with managed_progress(
                            id=f'launch_{launch.id}_{user.id}',
                            qty=cntAll,
                            user=user.id,
                            message=f'<h3>Формирование запуска: Заказ № {code} от {DateToStr(demand.date)}</h3>',
                            title='Выполнено',
                            props=TurnBitOn(0, 0)
                            # props=TurnBitOn(0, 1) #Без WebSocket progressbar a
                    ) as progress:

                        items_refs_stack = Item_refs_Stack()
                        items_refs_stack.add_parents(demand.precent_item.item.id)

                        for item_ref in Item_refs.objects.get_descendants(
                                id=demand.precent_item.item.id,
                                where_clause=where_clause
                        ):
                            if ~item_ref.props.used:
                                continue

                            items_refs_stack.push(item_ref)

                            try:
                                # start_time = time.clock()
                                item_line = Item_line.objects.get(parent=item_ref.parent, child=item_ref.child)
                            except Item_line.DoesNotExist:
                                raise Exception(blinkString(text=f'Для : PARENT_ID: {item_ref.parent.id if item_ref.parent else None} CHILD_ID: {item_ref.child.id} {items_refs_stack.get_full_path} не найдена строка детализации.', bold=True, blink=False))
                            # logger_timing.debug(f'Item_line.objects.get Time: {time.clock() - start_time}')

                            count = LaunchesManager.get_count(item_line=item_line, qty=qty, level=item_ref.level, get_full_path=items_refs_stack.get_full_path)

                            _count = StrToNumber(count.value_str) if count and item_line.section != 'Документация' else 0
                            # start_time = time.clock()
                            launch_item_refs, created = Launch_item_refs.objects.get_or_create(
                                child=item_ref.child,
                                parent=item_ref.parent if item_ref.level != 1 else None,
                                launch=launch,
                                qty=_count,
                                qty_per_one=round(_count / qty, 4),
                                defaults=dict(
                                    item_refs=item_ref,
                                    item_full_name=items_refs_stack.get_full_path,
                                    item_full_name_obj=items_refs_stack.get_full_path_obj
                                )
                            )
                            logger.debug(f'Created: {created} launch_item_refs :  {launch_item_refs}')
                            # logger_timing.debug(f'launch_item_refs Time: {time.clock() - start_time}')

                            # start_time = time.clock()
                            launch_item_line, created = Launch_item_line.objects.get_or_create(
                                child=item_line.child,
                                parent=item_line.parent,
                                item_line=item_line,
                                launch=launch,

                                SPC_CLM_FORMAT=item_line.SPC_CLM_FORMAT,
                                SPC_CLM_ZONE=item_line.SPC_CLM_ZONE,
                                SPC_CLM_POS=item_line.SPC_CLM_POS,
                                SPC_CLM_MARK=item_line.SPC_CLM_MARK,
                                SPC_CLM_NAME=item_line.SPC_CLM_NAME,
                                SPC_CLM_COUNT=count,
                                SPC_CLM_NOTE=item_line.SPC_CLM_NOTE,
                                SPC_CLM_MASSA=item_line.SPC_CLM_MASSA,
                                SPC_CLM_MATERIAL=item_line.SPC_CLM_MATERIAL,
                                SPC_CLM_USER=item_line.SPC_CLM_USER,
                                SPC_CLM_KOD=item_line.SPC_CLM_KOD,
                                SPC_CLM_FACTORY=item_line.SPC_CLM_FACTORY,
                                section=item_line.section,
                                section_num=item_line.section_num,
                                subsection=item_line.subsection,
                            )
                            logger.debug(f'Created : {created} launch_item_line :  {launch_item_line}')

                            # logger_timing.debug(f'launch_item_line Time: {time.clock() - start_time}')

                            def rec_operations_data(item, item_ref, get_full_path):
                                # start_time = time.clock()
                                cnt = Operations_item.objects.filter(item=item).count()
                                # logger_timing.debug(f'Operations_item.objects.filter Time: {time.clock() - start_time}')
                                if cnt == 0:
                                    if item_line.section != 'Документация':
                                        raise Exception(blinkString(text=f'Для : ID: {item.id} {get_full_path} не найдена операция.', bold=True, blink=False))
                                else:
                                    section = Item_line.objects.get(parent=item_ref.parent, child=item_ref.child).section

                                    if first_step == True and section and section == 'Документация':
                                        raise Exception(blinkString(text=f'Изделин : ID: {item.id} {get_full_path} должно входить как сборочная еденица.', bold=True, blink=False))

                                    if section and section != 'Документация':
                                        for operations_item in Operations_item.objects.filter(item=item):
                                            # start_time = time.clock()
                                            launch_operations_item, created = Launch_operations_item.objects.get_or_create(
                                                item=operations_item.item,
                                                launch=launch,
                                                num=operations_item.num,
                                                operation=operations_item.operation,
                                                defaults=dict(
                                                    description=operations_item.description,
                                                    ed_izm=operations_item.ed_izm,
                                                    operationitem=operations_item,
                                                    qty=operations_item.qty,
                                                )
                                            )
                                            logger.debug(f'Created: {created} launch_operations_item :  {launch_operations_item}')
                                            # logger_timing.debug(f'launch_operations_item Time: {time.clock() - start_time}')

                                            for operation_material in Operation_material.objects.filter(operationitem=operations_item):
                                                # start_time = time.clock()
                                                launch_operations_material, created = Launch_operations_material.objects.get_or_create(
                                                    launch_operationitem=launch_operations_item,
                                                    material=operation_material.material,
                                                    material_askon=operation_material.material_askon,
                                                    defaults=dict(
                                                        edizm=operation_material.edizm,
                                                        qty=operation_material.qty,
                                                        operation_material=operation_material,
                                                    )
                                                )
                                                logger.debug(f'Created: {created} launch_operations_material :  {launch_operations_material}')
                                                # logger_timing.debug(f'launch_operations_material Time: {time.clock() - start_time}')

                                            def exception_not_location():
                                                raise Exception(f'''<b>Для : {items_refs_stack.get_full_path}</b>  {blinkString(text='не задано местоположение. Запустите анализатор готовности к запуску.', blink=False, color='red', bold=True)}''')

                                            if Operation_resources.objects.filter(operationitem=operations_item).count() == 0:
                                                exception_not_location()

                                            for operation_resources in Operation_resources.objects.filter(operationitem=operations_item):
                                                if operation_resources.location == None:
                                                    exception_not_location()

                                                if operation_resources.resource == None:
                                                    # start_time = time.clock()
                                                    operation_resources.resource, created = settings.OPERS_STACK.NOT_UNDEFINED_WORKSHOP(operation_resources.location)
                                                    logger.debug(f'Created : {created} operation_resources.resource :  {operation_resources.resource}')
                                                    # logger_timing.debug(f'Resource.objects.get_or_create Time: {time.clock() - start_time}')

                                                # start_time = time.clock()
                                                launch_operation_resources, created = Launch_operation_resources.objects.get_or_create(
                                                    launch_operationitem=launch_operations_item,
                                                    resource=operation_resources.resource,
                                                    location=operation_resources.location,
                                                    defaults=dict(
                                                        operation_resources=operation_resources,
                                                        batch_size=operation_resources.batch_size,
                                                    )
                                                )
                                                logger.debug(f'Created: {created} launch_operation_resources.resource :  {launch_operation_resources}')
                                                # logger_timing.debug(f'Launch_operation_resources.objects.get_or_create Time: {time.clock() - start_time}')

                            if item_ref.parent:
                                if first_step == False:
                                    rec_operations_data(item=item_ref.parent, item_ref=item_ref, get_full_path=items_refs_stack.get_full_path)
                            rec_operations_data(item=item_ref.child, item_ref=item_ref, get_full_path=items_refs_stack.get_full_path)
                            first_step = False

                            step_res = progress.step()
                            if step_res != 0:
                                break

                        end_time = time.clock()
                        logger_timing.debug(f'End Time: {end_time}')
                        logger_timing.debug(f'Total Time: {end_time - start_time}')
                        if step_res == 0:
                            settings.EVENT_STACK.EVENTS_PRODUCTION_MAKE_LAUNCH.send_message(f'Выполнено формирование запуска  <h3>{launch.code} от {launch.date}</h3><p/>')

                if step_res != 0:
                    raise ProgressDroped(progress_deleted)
            settings.LOCKS.release('LaunchesManager.make_launch')
        except ProgressDroped as ex:
            Deleted_progresses.objects.filter(id_progress=progress.id, user=progress.user).delete()
            settings.LOCKS.release('LaunchesManager.make_launch')
            raise ex

        return data

    def createFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        setAttr(data, 'user_id', request.user_id)
        return LaunchesManager.make_launch(data)

    def reCreateFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        setAttr(data, 'user_id', request.user_id)
        return LaunchesManager.update_launch(data)

    def deleteFromRequest(self, request, removed=None, ):
        from isc_common.models.deleted_progresses import Deleted_progresses
        from kaf_pas.planing.models.operation_launches import Operation_launches
        from kaf_pas.production.models.launch_item_line import Launch_item_line
        from kaf_pas.production.models.launch_item_refs import Launch_item_refs
        from kaf_pas.production.models.launch_operation_material import Launch_operations_material
        from kaf_pas.production.models.launch_operation_resources import Launch_operation_resources
        from kaf_pas.production.models.launch_operations_item import Launch_operations_item

        request = DSRequest(request=request)
        res = 0
        tuple_ids = request.get_tuple_ids()
        step_res = 0

        try:
            with transaction.atomic():
                for id, mode in tuple_ids:
                    if mode == 'hide':
                        super().filter(id=id).soft_delete()
                    else:
                        launch = Launches.objects.get(id=id)

                        with managed_progress(
                                id=f'launch_{id}_{request.user_id}',
                                qty=Launch_operations_item.objects.filter(launch=id).count(),
                                user=request.user_id,
                                message=f'<h3>Удаление запуска: № {launch.code} от {DateToStr(launch.date)}</h3>',
                                title='Выполнено',
                                props=TurnBitOn(0, 0)
                        ) as progress:

                            qty, _ = Launch_item_refs.objects.filter(launch_id=id).delete()
                            qty1, _ = Launch_item_line.objects.filter(launch_id=id).delete()
                            qty += qty1

                            for launch_operations_item in Launch_operations_item.objects.filter(launch=id):
                                qty1, _ = Launch_operations_material.objects.filter(launch_operationitem=launch_operations_item).delete()
                                qty += qty1
                                qty1, _ = Launch_operation_resources.objects.filter(launch_operationitem=launch_operations_item).delete()
                                qty += qty1
                                launch_operations_item.delete()
                                qty += 1

                                for operation_launches in Operation_launches.objects.filter(launch_id=id):
                                    operation_launches.operation.delete()

                                step_res = progress.step()
                                if step_res != 0:
                                    break

                            if step_res == 0:
                                qty1, _ = super().filter(id=id).delete()

                        if step_res == 0:
                            settings.EVENT_STACK.EVENTS_PRODUCTION_DELETE_LAUNCH.send_message(f'Выполнено удаление запуска  <h3>{launch.code} от {launch.date}</h3><p/>')

                    if step_res != 0:
                        break
                    res += qty + qty1

            if step_res != 0:
                raise ProgressDroped(progress_deleted)
        except ProgressDroped as ex:
            Deleted_progresses.objects.filter(id_progress=progress.id, user=progress.user).delete()
            raise ex
        return res

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'code': record.code,
            'name': record.name,
            'date': record.date,
            'description': record.description,
            'parent_id': record.parent.id if record.parent else None,

            'demand_id': record.demand.id,
            'demand__code': record.demand.code,
            'demand__date': record.demand.date,

            'status_id': record.status.id,
            'status__code': record.status.code,
            'status__name': record.status.name,

            'qty': record.qty,
            'routing_made': record.props.routing_made,
            # 'props': record.props,

            'editing': record.editing,
            'deliting': record.deliting,
        }
        return DelProps(res)

    def get_queryset(self):
        return LaunchesQuerySet(self.model, using=self._db)


class Launches(BaseRefHierarcy):
    date = DateTimeField()
    status = ForeignKeyProtect(Status_launch)
    demand = ForeignKeyProtect(Demand)
    qty = PositiveIntegerField()
    # props = LaunchesManager.props()

    objects = LaunchesManager()

    def __str__(self):
        return f"ID:{self.id}, " \
               f"code: {self.code}, " \
               f"name: {self.name}, " \
               f"description: {self.description}, " \
               f"date: {self.date}, " \
               f"status: [{self.status}], " \
               f"demand: [{self.demand}], " \
               f"qty: {self.qty}"

    class Meta:
        verbose_name = 'Запуски'



from django.urls import path

from kaf_pas.planing.views import operation_standard_prod

urlpatterns = [

    path('Operation_standard_prod/Fetch/', operation_standard_prod.Operation_standard_prod_Fetch),
    path('Operation_standard_prod/Add', operation_standard_prod.Operation_standard_prod_Add),
    path('Operation_standard_prod/Update', operation_standard_prod.Operation_standard_prod_Update),
    path('Operation_standard_prod/Remove', operation_standard_prod.Operation_standard_prod_Remove),
    path('Operation_standard_prod/Lookup/', operation_standard_prod.Operation_standard_prod_Lookup),
    path('Operation_standard_prod/Info/', operation_standard_prod.Operation_standard_prod_Info),
    path('Operation_standard_prod/Copy', operation_standard_prod.Operation_standard_prod_Copy),

]



# Generated by Django 2.2 on 2019-04-15 13:07

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0020_item_refs'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='item',
            unique_together={('STMP_1', 'STMP_2')},
        ),
    ]



# Generated by Django 2.2.8 on 2019-12-12 20:56

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0173_auto_20191212_2041'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='item_refs',
            name='syntetic_parent',
        ),
    ]



from django.urls import path

from kaf_pas.ckk.views import locations

urlpatterns = [

    path('Locations/Fetch/', locations.Locations_Fetch),
    path('Locations/Add', locations.Locations_Add),
    path('Locations/Update', locations.Locations_Update),
    path('Locations/Remove', locations.Locations_Remove),
    path('Locations/Lookup/', locations.Locations_Lookup),
    path('Locations/Info/', locations.Locations_Info),

]



import logging
from typing import List

from django.db import transaction
from django.db.models import TextField, UniqueConstraint, Q, FloatField
from django.forms import model_to_dict

from isc_common import setAttr
from isc_common.common.functions import delete_dbl_spaces
from isc_common.fields.related import ForeignKeyProtect
from isc_common.http.DSRequest import DSRequest
from isc_common.logger.Logger import Logger
from isc_common.managers.common_managet_with_lookup_fields import CommonManagetWithLookUpFieldsManager, CommonManagetWithLookUpFieldsQuerySet
from isc_common.models.audit import AuditModel
from isc_common.number import DelProps, StrToNumber
from kaf_pas.ckk.models.attr_type import Attr_type, AttrManager

logger = logging.getLogger(__name__)
logger.__class__ = Logger


class Document_attributesQuerySet(CommonManagetWithLookUpFieldsQuerySet):

    def create(self, **kwargs):
        attr_type = kwargs.get('attr_type')
        if isinstance(attr_type, Attr_type):
            value_str = kwargs.get('value_str')
            if value_str != None:
                setAttr(kwargs, 'value_str', value_str.strip())
            return super().create(**kwargs)

        value_str = kwargs.get('value_str').strip()
        value_int = Document_attributesManager.to_int(value_str)

        if not kwargs.get('attr_type_id'):
            attr_type, _ = Attr_type.objects.get_or_create(code=kwargs.get('attr_type__code'))
            setAttr(kwargs, 'attr_type_id', attr_type.id)
        setAttr(kwargs, 'value_int', value_int)
        setAttr(kwargs, 'value_str', value_str)

        return super().create(**kwargs)

    def get_or_create(self, defaults=None, **kwargs):
        attr_type = kwargs.get('attr_type')
        if isinstance(attr_type, Attr_type):
            value_str = kwargs.get('value_str')
            if value_str != None:
                setAttr(kwargs, 'value_str', value_str.strip())
            return super().get_or_create(defaults, **kwargs)

        value_str = kwargs.get('value_str')
        if value_str != None:
            value_str = value_str.strip()

        if not kwargs.get('attr_type_id') and kwargs.get('attr_type__code'):
            attr_type, _ = Attr_type.objects.get_or_create(code=kwargs.get('attr_type__code'))
            setAttr(kwargs, 'attr_type_id', attr_type.id)
        elif isinstance(defaults, dict) and not defaults.get('attr_type_id') and defaults.get('attr_type__code'):
            attr_type, _ = Attr_type.objects.get_or_create(code=defaults.get('attr_type__code'))
            setAttr(kwargs, 'attr_type_id', attr_type.id)

        if not value_str and isinstance(defaults, dict):
            value_str = defaults.get('value_str').strip()
            value_int = Document_attributesManager.to_int(value_str)
            setAttr(defaults, 'value_int', value_int)
            setAttr(defaults, 'value_str', value_str)
        else:
            value_int = Document_attributesManager.to_int(value_str)
            setAttr(kwargs, 'value_int', value_int)
            setAttr(kwargs, 'value_str', value_str)

        return super().get_or_create(defaults, **kwargs)

    def update(self, **kwargs):
        attr_type = kwargs.get('attr_type')
        if isinstance(attr_type, Attr_type):
            return super().update(**kwargs)

        value_str = kwargs.get('value_str')
        value_int = Document_attributesManager.to_int(value_str)
        if not kwargs.get('attr_type_id'):
            attr_type, _ = Attr_type.objects.get_or_create(code=kwargs.get('attr_type__code'))
            setAttr(kwargs, 'attr_type_id', attr_type.id)
        setAttr(kwargs, 'value_int', value_int)

        return super().update(**kwargs)

    def update_or_create(self, defaults=None, **kwargs):
        attr_type = kwargs.get('attr_type')
        if isinstance(attr_type, Attr_type):
            return super().update_or_create(defaults, **kwargs)

        value_str = kwargs.get('value_str')

        if not kwargs.get('attr_type_id') and kwargs.get('attr_type__code'):
            attr_type, = Attr_type.objects.get_or_create(code=kwargs.get('attr_type__code'))
            setAttr(kwargs, 'attr_type_id', attr_type.id)
        elif isinstance(defaults, dict) and not defaults.get('attr_type_id') and defaults.get('attr_type__code'):
            attr_type, _ = Attr_type.objects.get_or_create(code=defaults.get('attr_type__code'))
            setAttr(kwargs, 'attr_type_id', attr_type.id)

        if not value_str and isinstance(defaults, dict):
            value_str = defaults.get('value_str')
            value_int = Document_attributesManager.to_int(value_str)
            setAttr(defaults, 'value_int', value_int)
        else:
            value_int = Document_attributesManager.to_int(value_str)
            setAttr(kwargs, 'value_int', value_int)

        return super().update_or_create(defaults, **kwargs)


class Document_attributesManager(CommonManagetWithLookUpFieldsManager):
    @staticmethod
    def get_or_create_attribute(attr_codes, value_str, attr_names=None, logger=None):
        from django.conf import settings
        settings.LOCKS.acquire('Document_attributesManager.get_or_create_attribute')
        attr_type = AttrManager.get_or_create_attr(attr_codes=attr_codes, attr_names=attr_names, logger=logger)
        value_int = StrToNumber(value_str)

        try:
            attribute = Document_attributes.objects.filter(
                value_str__delete_dbl_spaces=delete_dbl_spaces(value_str),
                attr_type=attr_type,
            ).order_by('id')[0]
            settings.LOCKS.release('Document_attributesManager.get_or_create_attribute')
            return attribute, False
        except IndexError:
            attribute = Document_attributes.objects.create(
                attr_type=attr_type,
                value_str=delete_dbl_spaces(value_str),
                value_int=value_int
            )
            settings.LOCKS.release('Document_attributesManager.get_or_create_attribute')
            return attribute, True

    def get_queryset(self):
        return Document_attributesQuerySet(self.model, using=self._db)

    @staticmethod
    def to_int(str):
        try:
            res = int(str)
            if res > 9223372036854775807:
                return None
            else:
                return res

        except:
            return None

    @staticmethod
    def getRecord(record):
        res = {
            "id": record.id,
            "attr_type_id": record.attr_type.id,
            "attr_type__code": record.attr_type.code,
            "attr_type__name": record.attr_type.name,
            "attr_type__description": record.attr_type.description,
            "value_str": record.value_str.strip(),
            "value_int": record.value_int,
            "lastmodified": record.lastmodified,
            "editing": record.editing,
            "deliting": record.deliting,
        }
        return res

    def update_document_attributes(self, document_attribute):
        with transaction.atomic():
            document_attributes = Document_attributes.objects.get(id=document_attribute.get('id'))
            document_attributes.value_str = document_attribute.get('value_str').strip()
            if document_attribute.get('value_int') != None:
                document_attributes.value_int = document_attribute.get('value_int')
            else:
                document_attributes.value_int = StrToNumber(document_attribute.get('value_str').strip())
            document_attributes.save()
            return document_attribute

    def updateFromRequest(self, request, removed=None, function=None):
        if not isinstance(request, DSRequest):
            request = DSRequest(request=request)
        data = request.get_data()
        return self.update_document_attributes(data)

    def createFromRequest(self, request, removed=None, propsArr: List = None):
        request = DSRequest(request=request)
        data = request.get_data()

        if data.get('value_int') != None and data.get('value_str') == None:
            setAttr(data, 'value_str', str(data.get('value_int')))
        elif data.get('value_int') == None and data.get('value_str') != None:
            setAttr(data, 'value_int', StrToNumber(data.get('value_str')))

        _data = data.copy()
        self._remove_prop(_data, removed)

        props = self.get_prp(data=_data, propsArr=propsArr)
        if props > -1:
            setAttr(_data, 'props', props)
        res = super().create(**_data)
        if data.get('value_int') != None:
            res.value_int = _data.get('value_int')
            res.save()

        res = model_to_dict(res)
        data.update(DelProps(res))
        return data

    def deleteFromRequest(self, request, removed=None, ):
        from kaf_pas.kd.models.document_attr_cross import Document_attr_cross

        request = DSRequest(request=request)
        res = 0
        tuple_ids = request.get_tuple_ids()
        with transaction.atomic():
            for id, mode in tuple_ids:
                if mode == 'hide':
                    super().filter(id=id).soft_delete()
                else:
                    Document_attr_cross.objects.filter(attribute_id=id).delete()
                    qty, _ = super().filter(id=id).delete()
                res += qty
        return res


class Document_attributes(AuditModel):
    attr_type = ForeignKeyProtect(Attr_type, verbose_name='Тип атрибута')
    value_str = TextField(verbose_name="Значение атрибута", db_index=True, null=True, blank=True)
    value_int = FloatField(verbose_name="Значение атрибута", db_index=True, null=True, blank=True)

    objects = Document_attributesManager()

    def __str__(self):
        return f'ID={self.id}, attr_type=[{self.attr_type}], value_str={self.value_str.strip()}, value_int={self.value_int}'

    class Meta:
        verbose_name = 'Аттрибуты докуменнта'
        constraints = [
            UniqueConstraint(fields=['attr_type', 'value_str'], condition=Q(value_int=None), name='Document_attributes_unique_constraint_0'),
            UniqueConstraint(fields=['attr_type', 'value_int', 'value_str'], name='Document_attributes_unique_constraint_1'),
        ]



# Generated by Django 3.0.4 on 2020-03-05 17:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0229_auto_20200305_1729'),
    ]

    operations = [
        migrations.AlterField(
            model_name='launch_item_refs',
            name='qty_per_one',
            field=models.FloatField(default=None),
        ),
        migrations.AddConstraint(
            model_name='resource',
            constraint=models.UniqueConstraint(condition=models.Q(code=None), fields=('location', 'name'), name='Resource_unique_constraint_1'),
        ),
    ]



# Generated by Django 2.2.7 on 2019-11-19 20:34

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0158_delete_status'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='locations',
            options={'verbose_name': 'Место положения'},
        ),
    ]



from django.urls import path

from kaf_pas.production.views import launch_item_view

urlpatterns = [

    path('Launch_item_view/Fetch/', launch_item_view.Launch_item_view_Fetch),
    path('Launch_item_view/Add', launch_item_view.Launch_item_view_Add),
    path('Launch_item_view/Update', launch_item_view.Launch_item_view_Update),
    path('Launch_item_view/Remove', launch_item_view.Launch_item_view_Remove),
    path('Launch_item_view/Lookup/', launch_item_view.Launch_item_view_Lookup),
    path('Launch_item_view/Info/', launch_item_view.Launch_item_view_Info),
    path('Launch_item_view/Copy', launch_item_view.Launch_item_view_Copy),

]



from django.urls import path

from kaf_pas.kd.views import documents_thumb10

urlpatterns = [

    path('Documents_thumb10/Fetch/', documents_thumb10.Documents_thumb10_Fetch),
    path('Documents_thumb10/Add', documents_thumb10.Documents_thumb10_Add),
    path('Documents_thumb10/Update', documents_thumb10.Documents_thumb10_Update),
    path('Documents_thumb10/Remove', documents_thumb10.Documents_thumb10_Remove),
    path('Documents_thumb10/Lookup/', documents_thumb10.Documents_thumb10_Lookup),
    path('Documents_thumb10/Info/', documents_thumb10.Documents_thumb10_Info),

]



# Generated by Django 2.2.3 on 2019-07-07 07:25

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0125_material_params_order'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='materials',
            name='material_1C',
        ),
    ]



import logging

from bitfield import BitField
from django.db import transaction
from django.db.models import UniqueConstraint, Q
from django.forms import model_to_dict

from isc_common import delAttr, setAttr
from isc_common.fields.related import ForeignKeyProtect, ForeignKeyCascade
from isc_common.http.DSRequest import DSRequest
from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet
from isc_common.number import DelProps
from kaf_pas.ckk.models.locations import Locations
from kaf_pas.production.models.operations import Operations
from kaf_pas.production.models.resource import Resource

logger = logging.getLogger(__name__)


class Operation_def_resourcesQuerySet(AuditQuerySet):
    def create(self, **kwargs):
        if kwargs.get('resource'):
            setAttr(kwargs, 'resource_id', kwargs.get('resource').id)
            delAttr(kwargs, 'resource')

        if kwargs.get('location'):
            setAttr(kwargs, 'location_id', kwargs.get('location').id)
            delAttr(kwargs, 'location')

        if not kwargs.get('resource_id') and not kwargs.get('location_id'):
            raise Exception('Необходим хотябы один выбранный параметр.')
        delAttr(kwargs, 'resource__name')
        delAttr(kwargs, 'location__full_name')

        resource_id = kwargs.get('resource_id')
        if resource_id:
            setAttr(kwargs, 'location_id', Resource.objects.get(id=resource_id).location.id)

        return super().create(**kwargs)

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Operation_def_resourcesManager(AuditManager):

    def allUpdateFromRequest(self, request):
        from kaf_pas.production.models.operations_item import Operations_item
        from kaf_pas.production.models.operation_resources import Operation_resources

        if not isinstance(request, DSRequest):
            request = DSRequest(request=request)
        data = request.get_data()
        qty = 0

        with transaction.atomic():
            for operation_def_resource in super().filter(id__in=data.get('ids')).select_for_update():
                for operation_item in Operations_item.objects.filter(operation=operation_def_resource.operation):
                    deleted, _ = Operation_resources.objects.filter(operationitem=operation_item).delete()
                    logger.debug(f'Deleted: {deleted}')
                    operation_resource, _ = Operation_resources.objects.get_or_create(
                        operationitem=operation_item,
                        resource=operation_def_resource.resource,
                        location=operation_def_resource.location,
                    )
                    logger.debug(f'Created: {operation_resource}')
                qty += 1

        return qty

    def updateFromRequest(self, request):
        if not isinstance(request, DSRequest):
            request = DSRequest(request=request)
        data = request.get_data()
        _data = data.copy()

        resource = None
        location = None

        if _data.get('resource_id') != None:
            resource = Resource.objects.get(id=_data.get('resource_id'))
            location = Locations.objects.get(id=resource.location.id)
            setAttr(_data, 'location_id', location.id)
            setAttr(_data, 'resource_id', resource.id)
        else:
            delAttr(_data, 'resource')

        if resource == None and _data.get('location_id') != None:
            location = Locations.objects.get(id=_data.get('location_id'))
            setAttr(_data, 'location_id', location.id)

        if resource == None and location == None:
            raise Exception('Необходим хотябы один выбранный параметр.')

        delAttr(_data, 'id')
        delAttr(_data, 'resource__code')
        delAttr(_data, 'resource__name')
        delAttr(_data, 'location__code')
        delAttr(_data, 'location__name')
        delAttr(_data, 'location__full_name')
        delAttr(_data, 'complex_name')
        delAttr(_data, 'rec_default')
        res, created = super().update_or_create(id=request.get_id(), defaults=_data)
        setAttr(data, 'id', res.id)
        return data

    def createFromRequest(self, request):
        from kaf_pas.production.models.operations_item import Operations_item
        from kaf_pas.production.models.operation_resources import Operation_resources

        request = DSRequest(request=request)
        data = request.get_data()
        _data = data.copy()
        rec_default = _data.get('rec_default')
        delAttr(_data, 'rec_default')
        operation_id = _data.get('operation_id')

        res = []

        def rec_defaults(operation, operation_resource):
            if rec_default:
                for operationitem in Operations_item.objects.filter(operation_id=operation):
                    if Operation_resources.objects.filter(operationitem=operationitem).count() == 0:
                        operation_resources, created = Operation_resources.objects.get_or_create(
                            operationitem=operationitem,
                            location=operation_resource.location,
                            resource=operation_resource.resource,
                        )
                        if created:
                            logger.debug(f'operation_resources: {operation_resources}')

        if isinstance(operation_id, list):
            delAttr(_data, 'operation_id')
            with transaction.atomic():
                for operation in operation_id:
                    setAttr(_data, 'operation_id', operation)
                    operation_resource = super().create(**_data)
                    _res = DelProps(model_to_dict(operation_resource))
                    setAttr(_res, 'location__full_name', operation_resource.location.full_name if operation_resource.location else None)
                    setAttr(_res, 'resource__name', operation_resource.resource.full_name if operation_resource.resource else None)
                    res.append(_res)

                    rec_defaults(operation=operation, operation_resource=operation_resource)
        else:
            with transaction.atomic():
                operation_resource = super().create(**_data)
                _res = DelProps(model_to_dict(operation_resource))
                setAttr(_res, 'location__full_name', operation_resource.location.full_name if operation_resource.location else None)
                setAttr(_res, 'resource__name', operation_resource.resource.full_name if operation_resource.resource else None)
                res.append(_res)
                rec_defaults(operation=operation_id, operation_resource=operation_resource)
        return res

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'operation_id': record.operation.id,
            'resource_id': record.resource.id if record.resource else None,
            'resource__code': record.resource.code if record.resource else None,
            'resource__name': record.resource.name if record.resource else None,
            'location_id': record.location.id if record.location else None,
            'location__code': record.location.code if record.location else None,
            'location__name': record.location.name if record.location else None,
            'location__full_name': record.location.full_name if record.location else None,
            'complex_name': record.complex_name,
            'editing': record.editing,
            'deliting': record.deliting,
            'rec_default': record.props.rec_default,
            'props': record.props,
        }
        return DelProps(res)

    @staticmethod
    def props():
        return BitField(flags=(
            ('rec_default', 'Записать ресурс по умолчанию на все операции на которых нет ресурсов'),  # 1
        ), default=0, db_index=True)

    def get_queryset(self):
        return Operation_def_resourcesQuerySet(self.model, using=self._db)


class Operation_def_resources(AuditModel):
    operation = ForeignKeyCascade(Operations)
    resource = ForeignKeyProtect(Resource, null=True, blank=True)
    location = ForeignKeyProtect(Locations, null=True, blank=True)
    props = Operation_def_resourcesManager.props()

    @property
    def complex_name(self):
        return f'{self.resource.location.full_name}{self.resource.name}' if self.resource else None

    objects = Operation_def_resourcesManager()

    def __str__(self):
        return f"ID: {self.id}"

    class Meta:
        verbose_name = 'Кросс таблица'
        constraints = [
            UniqueConstraint(fields=['operation'], condition=Q(location=None) & Q(resource=None), name='Operation_def_resources_unique_constraint_0'),
            UniqueConstraint(fields=['operation', 'resource'], condition=Q(location=None), name='Operation_def_resources_unique_constraint_1'),
            UniqueConstraint(fields=['location', 'operation'], condition=Q(resource=None), name='Operation_def_resources_unique_constraint_2'),
            UniqueConstraint(fields=['location', 'operation', 'resource'], name='Operation_def_resources_unique_constraint_3'),
        ]



from django.conf import settings

from isc_common import dictinct_list
from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.planing.models.production_order import Production_order, Production_orderManager
from kaf_pas.planing.models.production_order_detail import Production_order_detail, Production_order_detailManager


@JsonResponseWithException()
def Production_order_Fetch(request):
    opers_types = [
        settings.OPERS_TYPES_STACK.PRODUCTION_TASK.id,
    ]
    return JsonResponse(
        DSResponse(
            request=request,
            data=Production_order.objects.
                filter(
                opertype__in=opers_types,
            ).
                get_range_rows1(
                request=request,
                function=Production_orderManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Production_order_FetchDetail(request):
    opers_types = [
        settings.OPERS_TYPES_STACK.PRODUCTION_DETAIL_TASK.id,
    ]
    return JsonResponse(
        DSResponse(
            request=request,
            data=Production_order_detail.objects.
                filter(
                opertype__in=opers_types,
            ).
                get_range_rows1(
                request=request,
                function=Production_order_detailManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Production_order_FetchLocations(request):
    opers_types = [
        settings.OPERS_TYPES_STACK.PRODUCTION_TASK.id,
        # settings.OPERS_TYPES_STACK.PRODUCTION_DETAIL_TASK.id,
    ]

    return JsonResponse(
        DSResponse(
            request=request,
            data=dictinct_list(Production_order.objects.
                filter(
                opertype__in=opers_types,
                props__in=[
                    Production_order.props.product_order_routing
                ]
            ).
                values('resource__location_id', 'resource__location__name').
                distinct().
                order_by('resource__location__name').
                get_range_rows1(
                request=request,
                function=Production_orderManager.getRecordLocations
            )),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Production_order_FetchLevels(request):
    opers_types = [
        settings.OPERS_TYPES_STACK.PRODUCTION_TASK.id,
        # settings.OPERS_TYPES_STACK.PRODUCTION_DETAIL_TASK.id,
    ]

    return JsonResponse(
        DSResponse(
            request=request,
            data=dictinct_list(Production_order.objects.
                filter(
                opertype__in=opers_types,
                props__in=[
                    Production_order.props.product_order_routing,
                ]
            ).
                order_by('operation_level__code').
                values('operation_level_id', 'operation_level__name', 'operation_level__code').
                distinct().
                get_range_rows1(
                request=request,
                function=Production_orderManager.getRecordLevels
            )),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Production_order_FetchExecutorsLocation(request):
    from kaf_pas.ckk.models.locations_users import Locations_users
    from kaf_pas.ckk.models.locations_users import Locations_usersManager
    return JsonResponse(
        DSResponse(
            request=request,
            data=Locations_users.objects.
                filter().
                distinct().
                get_range_rows1(
                request=request,
                function=Locations_usersManager.getRecord1
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Production_order_Add(request):
    return JsonResponse(DSResponseAdd(data=Production_order.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Production_order_Update(request):
    return JsonResponse(DSResponseUpdate(data=Production_order.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Production_order_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Production_order.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Production_order_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Production_order.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Production_order_Info(request):
    return JsonResponse(DSResponse(request=request, data=Production_order.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Production_order_CheckStatus(request):
    return JsonResponse(DSResponse(request=request, data=Production_order.objects.get_queryset().get_checkStatus(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Production_order_SetPrevStatus(request):
    return JsonResponse(DSResponse(request=request, data=Production_order.objects.get_queryset().get_setPrevStatus(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Production_order_SetStartStatus(request):
    return JsonResponse(DSResponse(request=request, data=Production_order.objects.get_queryset().get_setStartStatus(request=request), status=RPCResponseConstant.statusSuccess).response)\

@JsonResponseWithException()
def Production_order_getValue_made(request):
    return JsonResponse(DSResponse(request=request, data=Production_order.objects.get_queryset().getValue_made(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Production_order_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Production_order.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)



from django.urls import path

from kaf_pas.planing.views import operation_section

urlpatterns = [

    path('Operation_section/Fetch/', operation_section.Operation_section_Fetch),
    path('Operation_section/Add', operation_section.Operation_section_Add),
    path('Operation_section/Update', operation_section.Operation_section_Update),
    path('Operation_section/Remove', operation_section.Operation_section_Remove),
    path('Operation_section/Lookup/', operation_section.Operation_section_Lookup),
    path('Operation_section/Info/', operation_section.Operation_section_Info),
    path('Operation_section/Copy', operation_section.Operation_section_Copy),

]



import logging

from django.db.models import PositiveIntegerField

from isc_common.fields.related import ForeignKeyProtect, ForeignKeyCascade
from isc_common.managers.common_managet_with_lookup_fields import CommonManagetWithLookUpFieldsQuerySet, CommonManagetWithLookUpFieldsManager
from isc_common.models.audit import AuditModel
from kaf_pas.ckk.models.item import Item
from kaf_pas.sales.models.precent_dogovors import Precent_dogovors

logger = logging.getLogger(__name__)


class Precent_items_viewQuerySet(CommonManagetWithLookUpFieldsQuerySet):
    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Precent_items_viewManager(CommonManagetWithLookUpFieldsManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'item_id': record.item.id,
            'item__STMP_1_id': record.item.STMP_1.id if record.item.STMP_1 else None,
            'item__STMP_1__value_str': record.item.STMP_1.value_str if record.item.STMP_1 else None,
            'item__STMP_2_id': record.item.STMP_2.id if record.item.STMP_2 else None,
            'item__STMP_2__value_str': record.item.STMP_2.value_str if record.item.STMP_2 else None,
            'precent_dogovor_id': record.precent_dogovor.id,
            'qty': record.qty,
            'used_qty': record.used_qty,
            'tail_qty': record.tail_qty,
            'lastmodified': record.lastmodified,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Precent_items_viewQuerySet(self.model, using=self._db)


class Precent_items_view(AuditModel):
    item = ForeignKeyProtect(Item)
    precent_dogovor = ForeignKeyCascade(Precent_dogovors)
    qty = PositiveIntegerField()
    used_qty = PositiveIntegerField()
    tail_qty = PositiveIntegerField()

    objects = Precent_items_viewManager()

    def __str__(self):
        return f'{self.id}'

    class Meta:
        verbose_name = 'Комплектация распоряжения'
        db_table = 'sales_precent_items_view'
        managed = False



from django.urls import path

from kaf_pas.kd.views import lotsman_documents_hierarcy_refs

urlpatterns = [

    path('Lotsman_documents_hierarcy_refs/Fetch/', lotsman_documents_hierarcy_refs.Lotsman_documents_hierarcy_refs_Fetch),
    path('Lotsman_documents_hierarcy_refs/Add', lotsman_documents_hierarcy_refs.Lotsman_documents_hierarcy_refs_Add),
    path('Lotsman_documents_hierarcy_refs/Update', lotsman_documents_hierarcy_refs.Lotsman_documents_hierarcy_refs_Update),
    path('Lotsman_documents_hierarcy_refs/Remove', lotsman_documents_hierarcy_refs.Lotsman_documents_hierarcy_refs_Remove),
    path('Lotsman_documents_hierarcy_refs/Lookup/', lotsman_documents_hierarcy_refs.Lotsman_documents_hierarcy_refs_Lookup),
    path('Lotsman_documents_hierarcy_refs/Info/', lotsman_documents_hierarcy_refs.Lotsman_documents_hierarcy_refs_Info),
    path('Lotsman_documents_hierarcy_refs/Copy', lotsman_documents_hierarcy_refs.Lotsman_documents_hierarcy_refs_Copy),

]



from django.urls import path

from kaf_pas.kd.views import documents_mview

urlpatterns = [

    path('Documents_mview/Fetch/', documents_mview.Documents_mview_Fetch),
    path('Documents_bad_mview/Fetch/', documents_mview.Documents_bad_mview_Fetch),
    path('Documents_mview/Add', documents_mview.Documents_mview_Add),
    path('Documents_mview/Update', documents_mview.Documents_mview_Update),
    path('Documents_mview/Remove', documents_mview.Documents_mview_Remove),
    path('Documents_mview/Lookup/', documents_mview.Documents_mview_Lookup),
    path('Documents_mview/Info/', documents_mview.Documents_mview_Info),
    path('Documents_bad_mview/Info/', documents_mview.Documents_mview_bad_Info),
    path('Documents_mview/RefreshMView', documents_mview.Documents_mview_RefreshMView),

]



# Generated by Django 2.2 on 2019-04-25 08:29

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('input', '0005_user_add_info'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Operation_plan_types',
        ),
    ]



# Generated by Django 2.2 on 2019-04-25 08:29

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0003_auto_20190424_1805'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='customer',
            options={'verbose_name': 'Покупатель'},
        ),
    ]



# Generated by Django 2.2.8 on 2019-12-15 01:08

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0109_auto_20191215_0106'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='lotsman_documents_hierarcy_refs',
            unique_together={('child', 'parent')},
        ),
    ]



# Generated by Django 2.2.8 on 2019-12-13 16:25

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import isc_common.fields.files
import isc_common.fields.name_field
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0096_auto_20191213_1334'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='lotsman_documents',
            name='upload',
        ),
        migrations.AddField(
            model_name='lotsman_documents',
            name='file_access_time',
            field=models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата время поcледнего доступа к документу'),
        ),
        migrations.AddField(
            model_name='lotsman_documents',
            name='file_change_time',
            field=models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата время изменнения документа'),
        ),
        migrations.AddField(
            model_name='lotsman_documents',
            name='file_document',
            field=models.TextField(db_index=True, default=None, verbose_name='Полный путь к файлу'),
        ),
        migrations.AddField(
            model_name='lotsman_documents',
            name='file_modification_time',
            field=models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата время поcледнего модификации документа'),
        ),
        migrations.AddField(
            model_name='lotsman_documents',
            name='file_size',
            field=models.PositiveIntegerField(default=None, verbose_name='Размер файла'),
        ),
        migrations.AlterField(
            model_name='lotsman_documents',
            name='id',
            field=models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор'),
        ),
        migrations.CreateModel(
            name='Lotsman_documents_thumb10',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('format', isc_common.fields.name_field.NameField(blank=True, null=True, verbose_name='Формат файла')),
                ('mime_type', isc_common.fields.name_field.NameField(blank=True, default=None, null=True, verbose_name='MIME тип файла файла')),
                ('size', models.BigIntegerField(default=0, verbose_name='Размер фала')),
                ('real_name', models.TextField(blank=True, default=None, null=True, verbose_name='Первоначальное имя файла')),
                ('key', models.BinaryField(blank=True, max_length=200, null=True)),
                ('attfile', isc_common.fields.files.FileFieldEx(blank=True, default=None, max_length=255, null=True, upload_to='', verbose_name='Файл')),
                ('file_store', models.CharField(blank=True, max_length=255, null=True, verbose_name='Каталог хранения файла')),
                ('path', models.TextField()),
                ('document', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='kd.Lotsman_documents', verbose_name='КД')),
            ],
            options={
                'verbose_name': 'JPEG варианты документов',
            },
        ),
        migrations.CreateModel(
            name='Lotsman_documents_thumb',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('format', isc_common.fields.name_field.NameField(blank=True, null=True, verbose_name='Формат файла')),
                ('mime_type', isc_common.fields.name_field.NameField(blank=True, default=None, null=True, verbose_name='MIME тип файла файла')),
                ('size', models.BigIntegerField(default=0, verbose_name='Размер фала')),
                ('real_name', models.TextField(blank=True, default=None, null=True, verbose_name='Первоначальное имя файла')),
                ('key', models.BinaryField(blank=True, max_length=200, null=True)),
                ('attfile', isc_common.fields.files.FileFieldEx(blank=True, default=None, max_length=255, null=True, upload_to='', verbose_name='Файл')),
                ('file_store', models.CharField(blank=True, max_length=255, null=True, verbose_name='Каталог хранения файла')),
                ('path', models.TextField()),
                ('document', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='kd.Lotsman_documents', verbose_name='КД')),
            ],
            options={
                'verbose_name': 'JPEG варианты документов уменьшенная копия',
            },
        ),
    ]



# Generated by Django 2.2.5 on 2019-09-18 02:09

import django.utils.timezone
from django.db import migrations, models

import isc_common.fields.code_field
import isc_common.fields.description_field
import isc_common.fields.name_field
import isc_common.fields.related


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Operation_plan_types',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('code', isc_common.fields.code_field.CodeField(blank=True, null=True)),
                ('name', isc_common.fields.name_field.NameField(blank=True, null=True)),
                ('description', isc_common.fields.description_field.DescriptionField()),
            ],
            options={
                'verbose_name': 'Типи операций планирования',
            },
        ),
        migrations.CreateModel(
            name='OperationsPlan',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('opertype', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='planing.Operation_plan_types')),
            ],
            options={
                'verbose_name': 'Операции планировая',
            },
        ),
        migrations.CreateModel(
            name='Operations_plan_view',
            fields=[
                ('operationsplan_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='planing.OperationsPlan')),
                ('isFolder', models.BooleanField()),
            ],
            options={
                'verbose_name': 'Операции планировая',
                'db_table': 'production_operationsplan_view',
                'managed': False,
            },
            bases=('planing.operationsplan',),
        ),
        migrations.CreateModel(
            name='Operations_plan_refs',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('child', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='child_Operations_plan', to='planing.OperationsPlan')),
                ('parent', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='parent_Operations_plan', to='planing.OperationsPlan')),
            ],
            options={
                'verbose_name': 'Кросс таблица',
            },
        ),
        migrations.CreateModel(
            name='Operations_plan_qtys',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('quantity', models.DecimalField(decimal_places=8, max_digits=20)),
                ('operations_plan', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='planing.OperationsPlan')),
            ],
            options={
                'verbose_name': 'Суммы',
            },
        ),
    ]



# Generated by Django 2.2.2 on 2019-06-14 19:47

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0105_auto_20190613_2223'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='item_production_specification',
            name='item',
        ),
        migrations.DeleteModel(
            name='Item_production_specificationView',
        ),
        migrations.DeleteModel(
            name='Item_production_specification',
        ),
    ]



# Generated by Django 3.0.3 on 2020-03-01 05:11

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0120_operations_num'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='production_order',
            options={'verbose_name': 'Заказы на производство'},
        ),
        migrations.AlterModelTable(
            name='production_order',
            table='production_operations_view',
        ),
    ]



from django.urls import path

from kaf_pas.production.views import operations_template_resource

urlpatterns = [

    path('Operations_template_resource/Fetch/', operations_template_resource.Operations_template_resource_Fetch),
    path('Operations_template_resource/Add', operations_template_resource.Operations_template_resource_Add),
    path('Operations_template_resource/Update', operations_template_resource.Operations_template_resource_Update),
    path('Operations_template_resource/Remove', operations_template_resource.Operations_template_resource_Remove),
    path('Operations_template_resource/Lookup/', operations_template_resource.Operations_template_resource_Lookup),
    path('Operations_template_resource/Info/', operations_template_resource.Operations_template_resource_Info),
    path('Operations_template_resource/Copy', operations_template_resource.Operations_template_resource_Copy),

]



from django.urls import path

from kaf_pas.production.views import launch_operation_resources

urlpatterns = [

    path('Launch_operation_resources/Fetch/', launch_operation_resources.Launch_operation_resources_Fetch),
    path('Launch_operation_resources/Add', launch_operation_resources.Launch_operation_resources_Add),
    path('Launch_operation_resources/Update', launch_operation_resources.Launch_operation_resources_Update),
    path('Launch_operation_resources/Remove', launch_operation_resources.Launch_operation_resources_Remove),
    path('Launch_operation_resources/Lookup/', launch_operation_resources.Launch_operation_resources_Lookup),
    path('Launch_operation_resources/Info/', launch_operation_resources.Launch_operation_resources_Info),
    path('Launch_operation_resources/Copy', launch_operation_resources.Launch_operation_resources_Copy),

]



from django.urls import path

from kaf_pas.kd.views import uploades_documents

urlpatterns = [

    path('Uploades_documents/Fetch/', uploades_documents.Uploades_documents_Fetch),
    path('Uploades_documents/Add', uploades_documents.Uploades_documents_Add),
    path('Uploades_documents/Update', uploades_documents.Uploades_documents_Update),
    path('Uploades_documents/Remove', uploades_documents.Uploades_documents_Remove),
    path('Uploades_documents/Lookup/', uploades_documents.Uploades_documents_Lookup),
    path('Uploades_documents/Info/', uploades_documents.Uploades_documents_Info),
    path('Uploades_documents/Copy', uploades_documents.Uploades_documents_Copy),

]



import logging

from django.core.management import BaseCommand
from django.db import transaction
from tqdm import tqdm

from kaf_pas.ckk.models.item import Item

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = "Заполнение поля 'Подтвержден' "

    def handle(self, *args, **options):
        logger.info(self.help)

        query = Item.objects.filter(props=~Item.props.confirmed)
        self.pbar = tqdm(total=query.count())

        with transaction.atomic():
            for item in query:
                item.props |= Item.props.confirmed
                item.save()
                self.pbar.update()

        self.pbar.close()



from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.production.models.launch_operation_resources import Launch_operation_resources, Launch_operation_resourcesManager


@JsonResponseWithException()
def Launch_operation_resources_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Launch_operation_resources.objects.
                filter().
                get_range_rows1(
                request=request,
                function=Launch_operation_resourcesManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Launch_operation_resources_Add(request):
    return JsonResponse(DSResponseAdd(data=Launch_operation_resources.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Launch_operation_resources_Update(request):
    return JsonResponse(DSResponseUpdate(data=Launch_operation_resources.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Launch_operation_resources_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Launch_operation_resources.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Launch_operation_resources_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Launch_operation_resources.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Launch_operation_resources_Info(request):
    return JsonResponse(DSResponse(request=request, data=Launch_operation_resources.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Launch_operation_resources_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Launch_operation_resources.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)



# Generated by Django 2.2.2 on 2019-06-08 13:41

import bitfield.models
from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0036_auto_20190607_1929'),
    ]

    operations = [
        migrations.CreateModel(
            name='Document_attrs_view',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('value_str', models.TextField(db_index=True, verbose_name='Значение атрибута')),
                ('props', bitfield.models.BitField((('relevant', 'Актуальность'), ('copied', 'Копированный')), db_index=True, default=1)),
            ],
            options={
                'verbose_name': 'Аттрибуты докуменнта',
                'db_table': 'kd_document_arrts_view',
                'managed': False,
            },
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-06 15:26

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0128_auto_20200206_1447'),
    ]

    operations = [
    ]



import logging

from isc_common.fields.related import ForeignKeyProtect, ForeignKeyCascade
from isc_common.models.audit import AuditManager, AuditModel, AuditQuerySet
from kaf_pas.planing.models.operations import Operations
from kaf_pas.production.models.resource import Resource

logger = logging.getLogger(__name__)


class Operation_resourcesQuerySet(AuditQuerySet):
    pass


class Operation_resourcesManager(AuditManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Operation_resourcesQuerySet(self.model, using=self._db)


class Operation_resources(AuditModel):
    operation = ForeignKeyCascade(Operations, related_name='planing_operation_res')
    resource = ForeignKeyProtect(Resource, related_name='planing_resource_res')

    objects = Operation_resourcesManager()

    def __str__(self):
        return f"ID:{self.id}, operation: [{self.operation}], resource: [{self.resource}]"

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Кросс таблица'
        unique_together = (('operation', 'resource'),)



# Generated by Django 2.2.6 on 2019-10-10 20:32

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0153_auto_20191010_1500'),
    ]

    operations = [
        migrations.AlterField(
            model_name='item_image_refs',
            name='thumb',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='kd.Documents_thumb'),
        ),
        migrations.AlterField(
            model_name='item_image_refs',
            name='thumb10',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='kd.Documents_thumb10'),
        ),
    ]



# Generated by Django 3.0.2 on 2020-01-22 07:50

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('accounting', '0008_tmp_buffer_color'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='tmp_buffer',
            name='color',
        ),
    ]



# Generated by Django 2.2 on 2019-04-25 19:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0007_auto_20190425_1915'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='operation_plan_template_locations',
            options={'verbose_name': 'Место положение для шаблонов заданий в производство'},
        ),
        migrations.AlterField(
            model_name='operation_plan_template_locations',
            name='distribution_coefficient',
            field=models.IntegerField(default=1, verbose_name='Коэффициент распределения'),
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-21 15:45

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0102_auto_20200219_1446'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='operations_view',
            options={},
        ),
    ]



# Generated by Django 2.2.8 on 2019-12-17 13:31

from django.db import migrations
import isc_common.fields.code_field


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0180_auto_20191217_1217'),
    ]

    operations = [
        migrations.AlterField(
            model_name='item_line',
            name='section',
            field=isc_common.fields.code_field.CodeField(blank=True, null=True),
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-28 20:49

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0042_auto_20190528_1710'),
    ]

    operations = [
        migrations.AlterField(
            model_name='precent',
            name='parent',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='parent_p', to='sales.Precent'),
        ),
        migrations.AlterUniqueTogether(
            name='precent_items',
            unique_together=set(),
        ),
    ]



import logging

from django.db.models import TextField
from django.forms import model_to_dict

from isc_common.fields.code_field import CodeStrictField
from isc_common.http.DSRequest import DSRequest
from isc_common.models.base_ref import BaseRefManager, BaseRefQuerySet, BaseRefHierarcy
from isc_common.number import DelProps

logger = logging.getLogger(__name__)


class ContantsQuerySet(BaseRefQuerySet):
    pass


class ContantsManager(BaseRefManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'code': record.code,
            'name': record.name,
            'description': record.description,
            'value': record.value,
            'parent_id': record.parent.id if record.parent else None,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return ContantsQuerySet(self.model, using=self._db)

    def createFromRequest(self, request, removed=None):
        request = DSRequest(request=request)
        data = request.get_data()
        _data = data.copy()
        self._remove_prop(_data, removed)

        res = super().create(**_data)
        res = model_to_dict(res)
        data.update(DelProps(res))
        return data

    # def updateFromRequest(self, request, removed=None):
    #     request = DSRequest(request=request)
    #     data = request.get_data()
    #     _data = data.copy()
    #     self._remove_prop(_data, removed)
    #
    #     isFolder = _data.get('isFolder', False)
    #     delAttr(_data, 'isFolder')
    #     id = _data.get('id')
    #     delAttr(_data, 'id')
    #     res = super().update_or_create(id=id, defaults=_data)
    #     data.setdefault('isFolder', isFolder)
    #     return data

    def refreshMatViewFromRequest(self, request):
        from kaf_pas.kd.models.lotsman_documents_hierarcy_ext import Lotsman_documents_hierarcyManagerExt

        if not isinstance(request, DSRequest):
            request = DSRequest(request=request)
        data = request.get_data()

        Lotsman_documents_hierarcyManagerExt(user=request.user).make_mview()

        return data

    def fixed_num_in_operationsFromRequest(self, request, removed=None, ):
        from kaf_pas.production.models.operations_item import Operations_itemManager
        request = DSRequest(request=request)

        res = dict(cnt=Operations_itemManager.refresh_num1())
        return res


class Contants(BaseRefHierarcy):
    code = CodeStrictField(unique=True)
    value = TextField(null=True, blank=True, db_index=True)
    objects = ContantsManager()

    def __str__(self):
        return f'ID:{self.id}, code:{self.code}, name:{self.name}, description:{self.description}'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Системные константы'



# Generated by Django 2.2.4 on 2019-08-07 17:11

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0079_auto_20190806_1007'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='pathes',
            name='props',
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-28 12:11

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0219_auto_20200227_2136'),
    ]

    operations = [
        migrations.RemoveConstraint(
            model_name='resource',
            name='Resource_unique_constraint_0',
        ),
        # migrations.RemoveConstraint(
        #     model_name='resource',
        #     name='Resource_unique_constraint_1',
        # ),
    ]



# Generated by Django 2.2.6 on 2019-10-18 13:46

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0157_item_line_section_num'),
        ('sales', '0065_dogovors_view_precent_view'),
    ]

    operations = [
        migrations.AddField(
            model_name='precent_materials',
            name='edizm',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='ckk.Ed_izm'),
        ),
    ]



# Generated by Django 2.2.8 on 2019-12-08 09:00

from django.db import migrations, models
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0161_auto_20191208_0859'),
    ]

    operations = [
        migrations.AlterField(
            model_name='launch_operation_resources',
            name='launch_operationitem',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='production.Launch_operations_item'),
        ),
        migrations.AlterField(
            model_name='launch_operation_resources',
            name='location',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='ckk.Locations'),
        ),
        migrations.AlterField(
            model_name='launch_operation_resources',
            name='resource',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='production.Resource'),
        ),
        migrations.AlterField(
            model_name='operation_resources',
            name='operationitem',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='production.Operations_item'),
        ),
    ]



# Generated by Django 2.2.6 on 2019-10-18 11:59

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0157_item_line_section_num'),
        ('sales', '0063_auto_20191018_1149'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='precent_items',
            name='location',
        ),
        migrations.AddField(
            model_name='precent_materials',
            name='material_askon',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='ckk.Material_askon'),
        ),
        migrations.AlterField(
            model_name='precent_materials',
            name='material',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='ckk.Materials'),
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-30 11:55

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0040_auto_20190529_1341'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='operations',
            name='code',
        ),
        migrations.RemoveField(
            model_name='operations',
            name='description',
        ),
        migrations.RemoveField(
            model_name='operations',
            name='name',
        ),
    ]



# Generated by Django 2.2 on 2019-04-21 13:40

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0022_auto_20190421_1330'),
    ]

    operations = [
        migrations.CreateModel(
            name='Pathes_view',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('isFolder', models.BigIntegerField()),
                ('path', models.TextField(verbose_name='Путь')),
            ],
            options={
                'verbose_name': 'Пути нахождения документов',
                'db_table': 'kd_pathes_view',
                'managed': False,
            },
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-22 05:27

from django.db import migrations, models
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0210_auto_20200212_0752'),
        ('planing', '0103_auto_20200221_1545'),
    ]

    operations = [
        # migrations.AddField(
        #     model_name='operations_view',
        #     name='material',
        #     field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='ckk.Material_askon'),
        # ),
        # migrations.AddField(
        #     model_name='operations_view',
        #     name='production_operation_edizm',
        #     field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='Operations_view_ed_izm', to='ckk.Ed_izm'),
        # ),
        # migrations.AddField(
        #     model_name='operations_view',
        #     name='production_operation_num',
        #     field=models.PositiveIntegerField(blank=True, null=True),
        # ),
        # migrations.AddField(
        #     model_name='operations_view',
        #     name='production_operation_qty',
        #     field=models.PositiveIntegerField(blank=True, null=True),
        # ),
    ]



# Generated by Django 2.2.3 on 2019-07-09 01:46

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0129_auto_20190708_1544'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='materials',
            name='ed_izm',
        ),
    ]



# Generated by Django 2.2.8 on 2019-12-06 11:34

import bitfield.models
from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0022_auto_20191206_0923'),
    ]

    operations = [
        migrations.CreateModel(
            name='Operations',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
            ],
            options={
                'verbose_name': 'Опреации учавствующие в планировании',
            },
        ),
        migrations.AlterField(
            model_name='operation_types',
            name='props',
            field=bitfield.models.BitField((('isoper', 'Является операцией'), ('plus', 'Приходная операция'), ('minus', 'Расходная операция')), db_index=True, default=0),
        ),
    ]



# Generated by Django 2.2.1 on 2019-06-05 15:33

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0071_auto_20190605_1416'),
    ]

    operations = [
        migrations.AlterField(
            model_name='item',
            name='document',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='kd.Documents', verbose_name='Документ'),
        ),
    ]



import logging

import requests
from django.conf import settings
from django.http import HttpResponseRedirect

from isc_common.http.DSRequest import DSRequest
from isc_common.http.DSResponse import DSResponseUpdate, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.kd.models.uploades import Uploades, UploadesManager

logger = logging.getLogger(__name__)

@JsonResponseWithException()
def Uploades_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Uploades.objects.
                filter().
                get_range_rows1(
                request=request,
                function=UploadesManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Uploades_Add(request):
    ws_channel = request.session.get('ws_channel')
    ws_port = settings.WS_PORT
    host = settings.WS_HOST

    _request = DSRequest(request=request)
    data = _request.get_data()

    r = requests.get(f'{settings.KOMPAS_INFORMICA}/logic/Uploades/Add', params=dict(path_id=data.get('path_id'), port=ws_port, ws_channel=ws_channel, host=host))
    HttpResponseRedirect(r.url)
    return JsonResponse(DSResponse(request=request, status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Uploades_Update(request):
    return JsonResponse(DSResponseUpdate(data=Uploades.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Uploades_Remove(request):
    ws_channel = request.session.get('ws_channel')
    ws_port = settings.WS_PORT
    host = settings.WS_HOST

    _request = DSRequest(request=request)
    ids = _request.get_tuple_ids()
    r = requests.get(f'{settings.KOMPAS_INFORMICA}/logic/Uploades/Remove', params=dict(ids=ids, port=ws_port, ws_channel=ws_channel, host=host))
    HttpResponseRedirect(r.url)
    return JsonResponse(DSResponse(request=request, status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Uploades_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Uploades.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Uploades_Info(request):
    return JsonResponse(DSResponse(request=request, data=Uploades.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Uploades_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Uploades.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Uploades_Confirmation(request):
    return JsonResponse(DSResponse(request=request, data=Uploades.objects.confirmationFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)

@JsonResponseWithException()
def Uploades_UnConfirmation(request):
    return JsonResponse(DSResponse(request=request, data=Uploades.objects.unConfirmationFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)



# Generated by Django 2.2.5 on 2019-09-18 14:37

import django.db.models.deletion
from django.db import migrations

import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0002_auto_20190918_1428'),
    ]

    operations = [
        migrations.AlterField(
            model_name='operationsplan',
            name='status',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='planing.OperationsPlanStatus'),
        ),
    ]



from django.conf import settings

from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.planing.models.operations import Operations
from kaf_pas.planing.models.operations_view import Operations_viewManager
from kaf_pas.planing.models.write_off_items import Write_off_items


@JsonResponseWithException()
def Write_off_items_Fetch(request):
    opers_types = [
        settings.OPERS_TYPES_STACK.WRITE_OFF_TASK.id,
        settings.OPERS_TYPES_STACK.WRITE_DETAIL_OFF_TASK.id
    ]

    return JsonResponse(
        DSResponse(
            request=request,
            data=Write_off_items.objects.
                filter(opertype__in=opers_types).
                get_range_rows1(
                request=request,
                function=Operations_viewManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Write_off_items_Add(request):
    return JsonResponse(DSResponseAdd(data=Write_off_items.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Write_off_items_Update(request):
    return JsonResponse(DSResponseUpdate(data=Operations.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Write_off_items_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Operations.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Write_off_items_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Write_off_items.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Write_off_items_Info(request):
    return JsonResponse(DSResponse(request=request, data=Write_off_items.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Write_off_items_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Write_off_items.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)



# Generated by Django 2.1.7 on 2019-03-27 07:03

import bitfield.models
import django.utils.timezone
from django.db import migrations, models

import isc_common.fields.code_field
import isc_common.fields.name_field
import isc_common.fields.related


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ('ckk', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Document_attrs',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted', models.BooleanField(db_index=True, default=False, verbose_name='Признак удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('section', isc_common.fields.code_field.CodeField(blank=True, null=True, verbose_name='Раздел')),
                ('subsection', isc_common.fields.code_field.CodeField(blank=True, null=True, verbose_name='Подраздел')),
                ('value_str', models.TextField(blank=True, null=True, verbose_name='Значение атрибута')),
                ('props', bitfield.models.BitField((('relevant', 'Актуальность'),), blank=True, default=None, null=True)),
                ('attr_type', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='ckk.Attr_type', verbose_name='Тип атрибута')),
            ],
            options={
                'verbose_name': 'Аттрибуты докуменнта',
            },
        ),
        migrations.CreateModel(
            name='Documents',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted', models.BooleanField(db_index=True, default=False, verbose_name='Признак удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('file_document', models.TextField(verbose_name='Полный путь к файлу')),
                ('file_size', models.PositiveIntegerField(verbose_name='Размер файла')),
                ('file_modification_time', models.DateTimeField(blank=True, null=True, verbose_name='Дата время поcледнего модификации документа')),
                ('file_access_time', models.DateTimeField(blank=True, null=True, verbose_name='Дата время поcледнего доступа к документу')),
                ('file_change_time', models.DateTimeField(blank=True, null=True, verbose_name='Дата время изменнения документа')),
                ('props', bitfield.models.BitField((('relevant', 'Актуальность'), ('beenItemed', 'Был внесен в состав изделий')), default=1)),
                ('attr_type', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='ckk.Attr_type', verbose_name='Тип документа')),
            ],
            options={
                'verbose_name': 'Конструкторские документы',
            },
        ),
        migrations.CreateModel(
            name='Documents_thumb',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted', models.BooleanField(db_index=True, default=False, verbose_name='Признак удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('format', isc_common.fields.name_field.NameField(verbose_name='Формат файла')),
                ('mime_type', isc_common.fields.name_field.NameField(blank=True, default=None, null=True, verbose_name='MIME тип файла файла')),
                ('size', models.BigIntegerField(default=0, verbose_name='Размер фала')),
                ('real_name', models.TextField(blank=True, default=None, null=True, verbose_name='Первоначальное имя файла')),
                ('key', models.BinaryField(blank=True, max_length=200, null=True)),
                ('attfile', models.FileField(blank=True, default=None, max_length=255, null=True, upload_to='', verbose_name='Файл')),
                ('file_store', models.CharField(blank=True, max_length=255, null=True, verbose_name='Каталог хранения файла')),
                ('path', models.TextField(unique=True)),
                ('document', isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to='kd.Documents', verbose_name='КД')),
            ],
            options={
                'verbose_name': 'JPEG варианты документов уменьшенная копия',
            },
        ),
        migrations.CreateModel(
            name='Documents_thumb10',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted', models.BooleanField(db_index=True, default=False, verbose_name='Признак удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('format', isc_common.fields.name_field.NameField(verbose_name='Формат файла')),
                ('mime_type', isc_common.fields.name_field.NameField(blank=True, default=None, null=True, verbose_name='MIME тип файла файла')),
                ('size', models.BigIntegerField(default=0, verbose_name='Размер фала')),
                ('real_name', models.TextField(blank=True, default=None, null=True, verbose_name='Первоначальное имя файла')),
                ('key', models.BinaryField(blank=True, max_length=200, null=True)),
                ('attfile', models.FileField(blank=True, default=None, max_length=255, null=True, upload_to='', verbose_name='Файл')),
                ('file_store', models.CharField(blank=True, max_length=255, null=True, verbose_name='Каталог хранения файла')),
                ('path', models.TextField(unique=True)),
                ('document', isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to='kd.Documents', verbose_name='КД')),
            ],
            options={
                'verbose_name': 'JPEG варианты документов',
            },
        ),
        migrations.CreateModel(
            name='Pathes',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted', models.BooleanField(db_index=True, default=False, verbose_name='Признак удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('path', models.TextField(verbose_name='Путь')),
                ('parent', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='kd.Pathes')),
            ],
            options={
                'verbose_name': 'Пути нахождения документов',
            },
        ),
        migrations.AddField(
            model_name='documents',
            name='path',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='kd.Pathes', verbose_name='Путь к документу'),
        ),
        migrations.AddField(
            model_name='document_attrs',
            name='document',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='kd.Documents', verbose_name='Документ'),
        ),
        migrations.CreateModel(
            name='Cdw',
            fields=[
            ],
            options={
                'verbose_name': 'Чертеж',
                'proxy': True,
                'indexes': [],
            },
            bases=('kd.document_attrs',),
        ),
        migrations.CreateModel(
            name='Spw',
            fields=[
            ],
            options={
                'verbose_name': 'Спецификация',
                'proxy': True,
                'indexes': [],
            },
            bases=('kd.document_attrs',),
        ),
        migrations.AlterUniqueTogether(
            name='document_attrs',
            unique_together={('document', 'attr_type', 'section', 'subsection')},
        ),
    ]



# Generated by Django 3.0.2 on 2020-01-19 17:50

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0185_auto_20200117_1305'),
        ('planing', '0067_auto_20200119_1444'),
    ]

    operations = [
        migrations.AddField(
            model_name='operation_value',
            name='edizm',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='ckk.Ed_izm'),
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-21 18:53

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import isc_common.fields.code_field
import isc_common.fields.description_field
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0043_auto_20190521_1853'),
        ('sales', '0007_customer_description'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='customer',
            options={'verbose_name': 'Клиент'},
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-24 12:00

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0025_remove_materials_parent'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='operation_material',
            name='material',
        ),
        migrations.RemoveField(
            model_name='operation_material',
            name='operation',
        ),
        migrations.DeleteModel(
            name='Materials',
        ),
        migrations.DeleteModel(
            name='Operation_material',
        ),
    ]



# Generated by Django 2.1.7 on 2019-03-27 11:11

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='document_attrs',
            name='props',
        ),
        migrations.RemoveField(
            model_name='documents',
            name='props',
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-20 17:54

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0145_auto_20190820_1059'),
    ]

    operations = [
        migrations.CreateModel(
            name='Material_askon_mview',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('name', models.CharField(db_index=True, max_length=255, verbose_name='Информация в дерево материалов')),
                ('field1', models.CharField(blank=True, max_length=255, null=True, verbose_name='Материал.Наименование и документ')),
                ('field2', models.CharField(blank=True, max_length=255, null=True, verbose_name='Материал.Код')),
                ('field3', models.CharField(blank=True, max_length=255, null=True, verbose_name='Сортамент.Наименование и документ')),
                ('field4', models.CharField(blank=True, max_length=255, null=True, verbose_name='Экземпляр сортамента.Типоразмер')),
                ('field5', models.CharField(blank=True, max_length=255, null=True, verbose_name='Экземпляр сортамента.Обозначение')),
                ('field6', models.CharField(blank=True, max_length=255, null=True, verbose_name='Экземпляр сортамента.Статус')),
                ('field7', models.CharField(blank=True, max_length=255, null=True, verbose_name='Экземпляр сортамента.Код')),
                ('isFolder', models.BooleanField()),
            ],
            options={
                'verbose_name': 'Материалы из Асона',
                'db_table': 'ckk_material_askon_mview',
                'managed': False,
            },
        ),
    ]



# Generated by Django 3.0.5 on 2020-04-17 14:20

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0223_item_qty_view'),
    ]

    operations = [
        migrations.AlterField(
            model_name='item_qty',
            name='child',
            field=isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, related_name='Item_qty_child', to='ckk.Item'),
        ),
        migrations.AlterField(
            model_name='item_qty',
            name='parent',
            field=isc_common.fields.related.ForeignKeyCascade(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='Item_qty_parent', to='ckk.Item'),
        ),
    ]



from django.urls import path

from kaf_pas.kd.views import lotsman_document_attr_cross

urlpatterns = [

    path('Lotsman_document_attr_cross/Fetch/', lotsman_document_attr_cross.Lotsman_document_attr_cross_Fetch),
    path('Lotsman_document_attr_cross/Add', lotsman_document_attr_cross.Lotsman_document_attr_cross_Add),
    path('Lotsman_document_attr_cross/Update', lotsman_document_attr_cross.Lotsman_document_attr_cross_Update),
    path('Lotsman_document_attr_cross/Remove', lotsman_document_attr_cross.Lotsman_document_attr_cross_Remove),
    path('Lotsman_document_attr_cross/Lookup/', lotsman_document_attr_cross.Lotsman_document_attr_cross_Lookup),
    path('Lotsman_document_attr_cross/Info/', lotsman_document_attr_cross.Lotsman_document_attr_cross_Info),
    path('Lotsman_document_attr_cross/Copy', lotsman_document_attr_cross.Lotsman_document_attr_cross_Copy),

]



import logging

from django.core.management import BaseCommand
from django.db import transaction, connection
from tqdm import tqdm

from kaf_pas.ckk.models.item import Item
from kaf_pas.ckk.models.item_refs import Item_refs
from kaf_pas.ckk.views.item_view import audo_top_level

logger = logging.getLogger(__name__)
logger1 = logging.getLogger(f'{__name__}_1')


class Command(BaseCommand):
    help = "Перенос чертежей в подпапку"

    def handle(self, *args, **options):

        logger.info(self.help)

        def get_sql_str(s='count(*)'):
            return f'''select {s}
                        from (
                                 (select *
                                 from ckk_item)
                                 except
                                 (select *
                                 from ckk_item
                                 where id in (select child_id from ckk_item_refs))
                             ) as a'''

        with connection.cursor() as cursor:
            print('Counting')

            cursor.execute(get_sql_str())
            cnt, = cursor.fetchone()
            print(f'Count: {cnt}')

        with transaction.atomic():
            print('Prepare')
            self.pbar = tqdm(total=cnt)
            for item in Item.objects.raw(get_sql_str(s='*')):
                Item_refs.objects.get_or_create(child=item, parent_id=int(audo_top_level))

                if self.pbar:
                    self.pbar.update()

        if self.pbar:
            self.pbar.close()



import logging

from isc_common.fields.related import ForeignKeyProtect, ForeignKeyCascade
from isc_common.managers.common_managet_with_lookup_fields import CommonManagetWithLookUpFieldsQuerySet, CommonManagetWithLookUpFieldsManager
from isc_common.models.audit import AuditModel
from kaf_pas.sales.models.demand import Demand
from kaf_pas.sales.models.precent_materials import Precent_materials

logger = logging.getLogger(__name__)


class Demand_materialsQuerySet(CommonManagetWithLookUpFieldsQuerySet):
    def create(self, **kwargs):
        return super().create(**kwargs)

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Demand_materialsManager(CommonManagetWithLookUpFieldsManager):

    @staticmethod
    def getRecord(record):
        res = {
            "id": record.id,
            'material_askon_id': record.precent_material.material_askon.id if record.precent_material.material_askon else None,
            'material_askon__field0': record.precent_material.material_askon.field0 if record.precent_material.material_askon else None,

            'material_id': record.precent_material.material.id if record.precent_material.material else None,
            'material__name': record.precent_material.material.name if record.precent_material.material else None,

            'complex_name': record.precent_material.complex_name,
            'complex_gost': record.precent_material.complex_gost,

            'edizm_id': record.precent_material.edizm.id,
            'edizm__code': record.precent_material.edizm.code,
            'edizm__name': record.precent_material.edizm.name,

            'qty': record.precent_material.qty,

            "lastmodified": record.lastmodified,
            "editing": record.editing,
            "deliting": record.deliting,
        }
        return res

    def get_queryset(self):
        return Demand_materialsQuerySet(self.model, using=self._db)


class Demand_materials(AuditModel):
    demand = ForeignKeyCascade(Demand)
    precent_material = ForeignKeyProtect(Precent_materials)

    @property
    def complex_name(self):
        if self.precent_material.material:
            return f'{self.precent_material.material.materials_type.full_name}{self.precent_material.material.full_name}'
        else:
            return None

    @property
    def complex_gost(self):
        if self.precent_material.material:
            if self.precent_material.material.materials_type.gost:
                return f'{self.precent_material.material.materials_type.gost} / {self.precent_material.material.gost}'
            else:
                return f'{self.precent_material.material.gost}'
        else:
            return None

    objects = Demand_materialsManager()

    def __str__(self):
        return f"{self.id}"

    class Meta:
        verbose_name = 'Кросс таблица'



# Generated by Django 2.2.2 on 2019-06-11 11:19

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import isc_common.fields.name_field
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0046_auto_20190610_0953'),
        ('ckk', '0090_auto_20190610_2018'),
    ]

    operations = [
        migrations.AlterModelTable(
            name='item_view',
            table='ckk_item_view',
        ),
        migrations.CreateModel(
            name='Cdw_Pdf_Item_view',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('STMP_1', isc_common.fields.name_field.NameField(verbose_name='Наименование изделия')),
                ('STMP_2', isc_common.fields.name_field.NameField(verbose_name='Обозначение изделия')),
                ('isFolder', models.BooleanField(verbose_name='Обозначение изделия')),
                ('parent', models.BigIntegerField(blank=True, null=True)),
                ('relevant', isc_common.fields.name_field.NameField()),
                ('props1', models.PositiveIntegerField(default=0)),
                ('document', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='kd.Documents', verbose_name='Документ')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]
